//*****************************************************************
//  
//  JAVA Source for com.idautomation.linear; 2006.9
//  
//  Copyright, IDAutomation.com, Inc. 2000-2006.
//  All rights reserved.
//  
//  http://www.IDAutomation.com/java/
//  
//  NOTICE:
//  You may incorporate our Source Code in your application
//  only if you own a valid Java Developer License
//  from IDAutomation.com, Inc. and the copyright notices 
//  are not removed from the source code.
//  
//*****************************************************************

package com.idautomation.linear;

import java.awt.*;
import java.awt.image.*;
import java.awt.image.MemoryImageSource;
import java.awt.image.PixelGrabber;
import java.io.PrintStream;
import java.beans.*;
import java.io.Serializable;

/* This is the main class that contains all linear barcode functionality. */

public class BarCode extends Canvas
implements Serializable {

	public final static int  CODE39=0;
	public final static int  CODE39EXT=1;
	public final static int  INTERLEAVED25=2;
	public final static int  CODE11=3;
	public final static int  CODABAR=4;
	public final static int  MSI=5;
	public final static int  UPCA=6;
	public final static int  IND25=7;
	public final static int  MAT25=8;
	public final static int  CODE93=9;
	public final static int  EAN13=10;
	public final static int  EAN8=11;
	public final static int  UPCE=12;
	public final static int  CODE128=13;
	public final static int  CODE93EXT=14;
	public final static int  POSTNET=15;
	public final static int  PLANET=16;
	public final static int  UCC128=17;
	public final static int  ONECODE=18;

/** This is the symbology of barcode. Default = CODE128. */

	public int barType=CODE128;


		static  int TABLE_2_OF_13_SIZE =   78;
		static  int TABLE_5_OF_13_SIZE = 1287;
		static boolean Table2of13InitializedFlag = false;
		static boolean Table5of13InitializedFlag = false;
		static long NumberOf5of13TableEntries=0L; 
		static long  NumberOf2of13TableEntries=0L; 
		static short  Remainder=0;
		static int[]  Table2of13 = new int[TABLE_2_OF_13_SIZE];
		static int[]  Table5of13= new int[TABLE_5_OF_13_SIZE];
		static int[]  BarTopCharacterIndexArray    = {  4, 0, 2, 6, 3,  5, 1, 9, 8, 7,  1, 2, 0, 6, 4,  8, 2, 9, 5, 3,  0, 1, 3, 7, 4,  6, 8, 9, 2, 0,  5, 1, 9, 4, 3,  8, 6, 7, 1, 2,  4, 3, 9, 5, 7,  8, 3, 0, 2, 1,  4, 0, 9, 1, 7,  0, 2, 4, 6, 3,  7, 1, 9, 5, 8 };
		static int[]  BarBottomCharacterIndexArray = {  7, 1, 9, 5, 8,  0, 2, 4, 6, 3,  5, 8, 9, 7, 3,  0, 6, 1, 7, 4,  6, 8, 9, 2, 5,  1, 7, 5, 4, 3,  8, 7, 6, 0, 2,  5, 4, 9, 3, 0,  1, 6, 8, 2, 0,  4, 5, 9, 6, 7,  5, 2, 6, 3, 8,  5, 1, 9, 8, 7,  4, 0, 2, 6, 3 };
		static int[]  BarTopCharacterShiftArray    = {  3, 0, 8,11, 1, 12, 8,11,10, 6,  4,12, 2, 7, 9,  6, 7, 9, 2, 8,  4, 0,12, 7,10,  9, 0, 7,10, 5,  7, 9, 6, 8, 2, 12, 1, 4, 2, 0,  1, 5, 4, 6,12,  1, 0, 9, 4, 7,  5,10, 2, 6, 9, 11, 2,12, 6, 7,  5,11, 0, 3, 2 };
		static int[]  BarBottomCharacterShiftArray = {  2,10,12, 5, 9,  1, 5, 4, 3, 9, 11, 5,10, 1, 6,  3, 4, 1,10, 0,  2,11, 8, 6, 1, 12, 3, 8, 6, 4,  4,11, 0, 6, 1,  9,11, 5, 3, 7,  3,10, 7,11, 8,  2,10, 3, 5, 8,  0, 3,12,11, 8,  4, 5, 1, 3, 0,  7,12, 9, 8,10 };
		static int[] Table5of13ArrayPtr = new int[TABLE_5_OF_13_SIZE];
		static int[] Table2of13ArrayPtr =new int[TABLE_2_OF_13_SIZE];
		//static NumberRecordType[] CodewordArray = new NumberRecordType[10];
		static double[][] CodewordArray = new double[10][2];
		int[] byteArray = new int[13];
/** To change the symbology type, set the appropriate number for the SymbologyID property:  CODE39=0; CODE39EXT=1; INTERLEAVED25=2; CODE11=3; CODABAR=4; MSI=5; UPCA=6; IND25=7; MAT25=8; CODE93=9; EAN13=10; EAN8=11; UPCE=12; CODE128=13; CODE93EXT=14; POSTNET=15; PLANET=16; UCC128=17; the default is CODE128.
 */
	public void setSymbologyID(int newSym) {
		barType = newSym;
		invalidate();
	}
/** Returns the symbology type; CODE39=0; CODE39EXT=1; INTERLEAVED25=2; CODE11=3; CODABAR=4; MSI=5; UPCA=6; IND25=7; MAT25=8; CODE93=9; EAN13=10; EAN8=11; UPCE=12; CODE128=13; CODE93EXT=14; POSTNET=15; PLANET=16; UCC128=17; the default is CODE128.
 */
	public int getSymbologyID() {
		return barType;
	}


/** The data to be encoded as a barcode. */

	//public String code="";
	public String code="123456789012";
	private String propCode = code;

/** Sets the data to be encoded as a barcode. */
	public void setDataToEncode(String newCode) {
		//propCode = newCode;
		code = newCode;
		invalidate();
	}

/** Gets the data to be encoded as a barcode. */
	public String getDataToEncode() {
		return code;
		//return propCode;
	}

	private String codeSup="";
	
	private int demoPixelW=0;

	private int codeTextW = 0;
/** The amount of space in CM between the barcode and the human readable text. Default is 0.3CM */
	public double textMarginCM =0.3;
	private int textMarginPixels =0;
	
	private int XdimPixels = 0;
	private int WhiteBarIncreasePixels = 0;
	float fXdimPixels = 0;
	
//TEXT MARGIN brj 6/22/06
/** Sets the textMarginCM property. */
	public void setTextMarginCM(double newMargin) {
			
		//if (newMargin < 0.3)
			//newMargin = 0.3;
			
		if (newMargin > 10)
			newMargin = 10;
	
	  //prevents an increase of text margin when showtext  (human readable is false)  prevents demo text from being moved out.
	  if (showText ==false)
	     {
	    newMargin = 0.3;
	    }
			
		textMarginCM = newMargin;
		invalidate();
	}
/** Returns the textMarginCM property value */
	public double getTextMarginCM() {
		return textMarginCM;
	}

/** If true, the checksum character will be calculated and appended to the code; Default =True; checksum characters cannot be displayed or disabled for Code 128 and UCC128. */
	public boolean checkCharacter=true;

/** If true, the checksum character will be calculated and appended to the code; Default =True; checksum characters cannot be displayed or disabled for Code 128 and UCC128. */
	public void setCheckCharacter(boolean newChk) {
		checkCharacter = newChk;
		invalidate();
	}
	
/** If true, the checksum character will be calculated and appended to the code; Default =True; checksum characters cannot by displayed or disabled for Code 128 and UCC128. */	
	public boolean getCheckCharacter() {
		return checkCharacter;
	}

/** If true, the checksum character will be appended to the human readable text; Default =True; checksum characters cannot by displayed or disabled for Code 128 and UCC128. */

	public boolean checkCharacterInText=true;

/** If true, the checksum character will be appended to the human readable text; Default =True; checksum characters cannot by displayed or disabled for Code 128 and UCC128. */
	public void setCheckCharacterInText(boolean newVal) {
		checkCharacterInText = newVal;
		invalidate();
	}
/** If true, the checksum character will be appended to the human readable text; Default =True; checksum characters cannot by displayed for Code 128. */
	public boolean getCheckCharacterInText() {
		return checkCharacterInText;
	}

	/** If true, numbers after the Tilde will be processed according to the rules in the user guide; this option is only available when the symbology is Code 128, and the character set is AUTO or 0. */
	public boolean processTilde=true;
	public boolean getProcessTilde() {return processTilde;}
	public void setProcessTilde(boolean pt) {processTilde=pt;}


/** Height in CM of POSTNET's tall bars; The default is 0.3 */	

	public double postnetHeightTallBar=(.3);

/** Height in CM of POSTNET's tall bars; The default is 0.3 */	

	public void setPostnetHeightTall(double newHeight) {
		postnetHeightTallBar = newHeight;
		invalidate();
	}
/** Returns the height in CM of POSTNET's tall bars; The default is 0.3 */	
	public double getPostnetHeightTall() {
		return postnetHeightTallBar;
	}

/** Height in CM of POSTNET's short bars; The default is 0.125 */

	public double postnetHeightShortBar=(.125);

/** Height in CM of POSTNET's short bars; The default is 0.125 */

	public void setPostnetHeightShort(double newHeight) {
		postnetHeightShortBar = newHeight;
		invalidate();
	}
/** Returns the height in CM of POSTNET's short bars; The default is 0.125 */
	public double getPostnetHeightShort() {
		return postnetHeightShortBar;
	}
	
/** The left margin of the barcode (also know as quiet zone); The default is .3 CM. */

	public double leftMarginCM=0.3;

/** The left margin of the barcode (also know as quiet zone); The default is .3 CM. */

	public void setLeftMarginCM(double newMargin) {
		//Added by TB 2/2006, if vertical bearer bars are in use the leftmargin is increased to 10 times the X dimension
		if (fBearerBarVert > 0 && newMargin < (10 * X))
			newMargin = 10 * X;

		leftMarginCM = newMargin;
		invalidate();
	}
/** Returns the left margin of the barcode (also know as quite zone); The default is .3 CM. */
	public double getLeftMarginCM() {
		return leftMarginCM;
	}

//***************************************** 1=sale version
//***************************************** 3=demo
	protected final static int d=3;
	

/** The top margin of the barcode (also know as quite zone); The default is .2 CM */

	public double topMarginCM=.2;

/** The top margin of the barcode (also know as quite zone); The default is .2 CM */
	
	public void setTopMarginCM(double newMargin) {
		topMarginCM = newMargin;
		invalidate();
	}
/** Returns the top margin of the barcode (also know as quite zone); The default is .2 CM */
	public double getTopMarginCM() {
		return topMarginCM;
	}

	protected int leftMarginPixels=0;
	protected int topMarginPixels=0;

	private int leftGuardBar=0;
	private int centerGuardBarStart=0;
	private int centerGuardBarEnd=0;
	private int rightGuardBar=0;
	private int endOfCode=0;
	
	private int startSuplement;
	private int endSuplement;
	private int suplementTopMargin;

/** This is the user defined 2 or 5 digit supplement for EAN or UPC codes. */
	public String supplement=""; // user defined UPC or EAN supplement

/** Sets the user defined 2 or 5 digit supplement for EAN or UPC codes. */
	public void setSupplementToEncode(String newCode) {
		//supplement = newCode;
		//tb 7/262006 format the supplement
			//can only be 2 or 5 digits long
		if(newCode.length() == 2)
		{
			UPCEANSupplement2 = true;
			UPCEANSupplement5 = false;
		}
		
		if(newCode.length() == 5)
		{
			UPCEANSupplement5 = true;
			UPCEANSupplement2 = false;
		}
		if(newCode.length() == 2 || newCode.length() == 5)
			supplement = newCode;
		else 
			supplement = "";
		invalidate();
	}
/** Gets the user defined 2 or 5 digit supplement for EAN or UPC codes. */
	public String getSupplementToEncode() {
		return supplement;
	}


/** If true (default), guardbars in EAN and UPC codes will be longer than data bars.
 */	
	public boolean guardBars=true;

/** Background color. */

  	public java.awt.Color backColor=java.awt.Color.white;

/** Sets the background color. */

	public void setBackground(Color newColor) {
		backColor = newColor;
		invalidate();
	}
/** Gets the background color. */
	public Color getBackground() {
		return backColor;
	}

/** The text of the barcode after it is created.
 */
	public String codeText="";

	protected int narrowBarPixels=0;
	protected int widthBarPixels=0;
	protected double narrowBarCM=0;
	protected double widthBarCM=0;
		
/** The resolution used to create the X dimension and other measurements in pixels; Default is 38 Pixels Per CM which is about 96 DPI. 96 DPI is the resolution of the web browser; this setting works best when using the servlet. Enter 118 to create a 300 DPI image.*/
    public int resolution=38;
    //public int resolution=(int) (this.getToolkit().getScreenResolution()/(2.54));

/** Sets the resolution used to convert from CM to pixels; Default is 38. */
	public void setPixelsPerCM(int newRes) {
		resolution = newRes;
		invalidate();
	}
/** Gets the resolution used to convert from CM to pixels; Default is 38. */
	public int getPixelsPerCM() {
		return resolution;
	}

	protected int barHeightPixels=0;


/** Bar height in CM; If 0 it will be calculated using H; Default=1.*/	

	public double barHeightCM=1;

/** Sets bar height in CM; If 0 it will be calculated using H; Default=1.*/	

	public void setBarHeightCM(double newBarHeight) {
		barHeightCM = newBarHeight;
		invalidate();
	}
/** Gets bar height in CM; If 0 it will be calculated using H; Default=1.*/
	public double getBarHeightCM() {
		return barHeightCM;
	}

/** Initial preferred width property of the bean */
	public int width=170;
	
/** Initial preferred height property of the bean */
	public int height=90;

/** The preferred width of the barcode image in pixels after barcode is generated. */
	public int pWidth = width;
	
	
/** The preferred height of the barcode image in pixels after barcode is generated. */
	public int pHeight = height;
	

//**** NEW CODE ENTERED HERE TO CALCULATE SIZE ***********
//********************************************************
//********************************************************
//********************************************************

/** If true will automatically size the control canvas to fit the barcode at design or runtime; Default =True. */
	public boolean autoSize=true;

/** If true will automatically size the control canvas to fit the barcode at design or runtime; Default =True. */

	public void setAutoSize(boolean newVal) {
		autoSize= newVal;
		invalidate();
	}
/** If true will automatically size the control canvas to fit the barcode at design or runtime; Default =True. */
	public boolean getAutoSize() {
		return autoSize;
	}

/** Returns the size in pixels of the Preferred barcode image generated. */
	public Dimension getPreferredSize() {
		return new Dimension(pWidth,pHeight);
	}

	public Dimension getMinimumSize() {
		Dimension dimensionMin = new Dimension(10,10);
		return dimensionMin;
	}

/** If true, the human readable text will be displayed with the barcode; Default =true. */
	public boolean showText=true;

/** If true, the human readable text will be displayed with the barcode; Default =true. */

	public void setShowText(boolean newVal) {
		showText= newVal;
		invalidate();
	}
/** If true, the human readable text will be displayed with the barcode; Default =true. */
	public boolean getShowText() {
		return showText;
	}

/** The font of the text in the barcode; Default = Arial. */
	//public Font textFont=new Font("Arial",Font.PLAIN,11);
	//tb, point size based on resolution
	public Font textFont=new Font("Arial",Font.PLAIN,11*(int)(resolution*.027));

/** Sets the font of the text. */
	public void setFont(Font newFont) {
		textFont = newFont;
		invalidate();
	}
	
/** Gets the font of the text. */
	public Font getFont() {
		return textFont;
	}	

/** The color of the text; Default=black. */
	public java.awt.Color fontColor=java.awt.Color.black;

/** Sets color of the text; Default=black. */
	public void setTextFontColor(Color newColor) {
		fontColor = newColor;
		invalidate();
	}
	
/** Gets color of the text; Default=black. */
	public Color getTextFontColor() {
		return fontColor;
	}

/** The color of the bars in the barcode;	Default=black. */
	public java.awt.Color barColor=java.awt.Color.black;

/** Sets color of the bars; Default=black. */
	public void setForeground(Color newColor) {
		barColor = newColor;
		invalidate();
	}
	
/** Gets color of the bars; Default=black. */
	public Color getForeground() {
		return barColor;
	}

	private int extraHeight=0;

/** The system to be used in UPCE; It can be "0" or "1"; The default is "0". */	

	public char UPCESytem ='0';

/** Sets the system to be used in UPCE; It can be "0", "1", or "2"; The default is "0". */
	public void setUPCESytem(String newSet) {
		if (newSet.equals("0")) UPCESytem= '0';
		if (newSet.equals("1")) UPCESytem= '1';
		//next line added by TB 6/2006, if 2 then auto system is used
		if (newSet.equals("2")) UPCESytem= '2';
		invalidate();
	}
/** Gets the system to be used in UPCE; It can be "0", "1", or "2". */
	public String getUPCESytem() {
		String setReturn = "";
		if (UPCESytem=='0') setReturn = "0";
		if (UPCESytem=='1') setReturn = "1";
		if (UPCESytem=='2') setReturn = "2";//auto
		return setReturn;
	}

/** The start character for CODABAR; The default is A. */

	public char CODABARStartChar='A';

/** Sets the start character for CODABAR; The default is A. */
	public void setCODABARStartChar(String newSet) {
		if (newSet.equals("B")) CODABARStartChar= 'B';
		if (newSet.equals("A")) CODABARStartChar= 'A';
		if (newSet.equals("C")) CODABARStartChar= 'C';
		if (newSet.equals("D")) CODABARStartChar= 'D';
		invalidate();
	}
/** Gets the start character for CODABAR. */
	public String getCODABARStartChar() {
		String setReturn = "";
		if (CODABARStartChar=='B') setReturn = "B";
		if (CODABARStartChar=='A') setReturn = "A";
		if (CODABARStartChar=='C') setReturn = "C";
		if (CODABARStartChar=='D') setReturn = "D";
		return setReturn;
	}

/** The stop character for CODABAR; The default is B. */

	public char CODABARStopChar='B';

/** Sets the stop character for CODABAR; The default is B. */
	public void setCODABARStopChar(String newSet) {
		if (newSet.equals("B")) CODABARStopChar= 'B';
		if (newSet.equals("A")) CODABARStopChar= 'A';
		if (newSet.equals("C")) CODABARStopChar= 'C';
		if (newSet.equals("D")) CODABARStopChar= 'D';
		invalidate();
	}
/** Gets the stop character for CODABAR; The default is B. */
	public String getCODABARStopChar() {
		String setReturn = "";
		if (CODABARStopChar=='B') setReturn = "B";
		if (CODABARStopChar=='A') setReturn = "A";
		if (CODABARStopChar=='C') setReturn = "C";
		if (CODABARStopChar=='D') setReturn = "D";
		return setReturn;
	}

/** If true, the 2 digit supplement for EAN or UPC codes is processed.
 */
	public boolean UPCEANSupplement2=false;
/** Sets the 2 digit supplement for EAN or UPC codes.
 */
	public void setUPCEANSupplement2(boolean newval) {
		UPCEANSupplement2 = newval;
		invalidate();
	}
/** Gets the 2 digit supplement for EAN or UPC codes.
 */
	public boolean getUPCEANSupplement2() {
		return UPCEANSupplement2;
	}

/** If true, the 5 digit supplement for EAN or UPC codes is processed.
 */
	public boolean UPCEANSupplement5=false;
/** Sets the 5 digit supplement for EAN or UPC codes.
 */
	public void setUPCEANSupplement5(boolean newval) {
		UPCEANSupplement5 = newval;
		invalidate();
	}
/** Gets the 5 digit supplement for EAN or UPC codes.
 */
	public boolean getUPCEANSupplement5() {
		return UPCEANSupplement5;
	}

/** The character set to be used in code 128; Possible values are "0 for AUTO", "A", "B" or "C"; The recommended setting and the default is "0" (Automatic)*/

	public char Code128Set='0';

/** Sets the character set to be used in code 128; Possible values are "0 for AUTO", "A", "B" or "C"; The recommended setting and the default is "0" (Automatic)*/
	public void setCode128Set(String newSet) {
		if ((newSet.toUpperCase()).equals("B")) Code128Set = 'B';
		if ((newSet.toUpperCase()).equals("A")) Code128Set = 'A';
		if ((newSet.toUpperCase()).equals("C")) Code128Set = 'C';
		if ((newSet.toUpperCase()).equals("AUTO")) Code128Set = '0';
		if ((newSet.toUpperCase()).equals("0")) Code128Set = '0';
		invalidate();
	}
/** Gets the character set to be used in code 128; Possible values are "0 for AUTO", "A", "B" or "C"; The recommended setting and the default is "0" (Automatic)*/
	public String getCode128Set() {
		String setReturn = "";
		if (Code128Set=='B') setReturn = "B";
		if (Code128Set=='A') setReturn = "A";
		if (Code128Set=='C') setReturn = "C";
		if (Code128Set=='0') setReturn = "0";
		return setReturn;
	}

/** The size in CM of modules (narrow bars or spaces); The resolution is used to convert to pixels. */

	public double X=0.030; // CM , X-dimension
	

	

/** Sets the size in CM of modules (narrow bars or spaces); The resolution is used to convert to pixels. */
	public void setXDimensionCM(double newX) {
		X = newX;
		invalidate();
	}
/** Gets the size in CM of modules (narrow bars or spaces); The resolution is used to convert to pixels. */
	public double getXDimensionCM() {
		return X;
	}

/** The wide to narrow ratio of barcode types that support this setting such as Code 39 and ITF; A value of 2 (default) means that wide bars will be 2*N in width. */

	public double N=2.0; // multiple value for width bars

/** Sets the wide to narrow ratio of barcode types that support this setting such as Code 39 and ITF; A value of 2 (default) means that wide bars will be 2*N in width. */	public void setNarrowToWideRatio(double newN) {
		N = newN;
		invalidate();
	}
/** Gets the wide to narrow ratio of barcode types that support this setting such as Code 39 and ITF; A value of 2 (default) means that wide bars will be 2*N in width. */
	public double getNarrowToWideRatio() {
		return N;
	}

/** The space between 2 characters in code 39; This a multiple of X; The default is 1.; */
	public double I=1; // space between 2 chars, multiple of X

/** The height of bars in the barcode as a multiple of X; The default is 0.45. */
	public double H=0.45; // height, multiple of the length

/** The symbol length in CM. */
	public double L=0; // symbol length

/** The rotation angle; Valid values are 0 (default), 90, 180 & 270.
 */
	public int rotate=0; // valid values are 0, 90, 180 & 270

/** Sets the rotation angle; Valid values are 0 (default), 90, 180 & 270.
 */
	public void setRotationAngle(int newRotate) {
		rotate = newRotate;
		invalidate();
	}
/** Gets the rotation angle; Valid values are 0 (default), 90, 180 & 270.
 */
	public int getRotationAngle() {
		return rotate;
	}	


	/** The separation in CM between the barcode and the supplement
	 */
	public double supSeparationCM=0.5;

	/** Height of the supplement; This is a multiplicator of the height of the code; The default is 0.8 (80%).
	 */
	public double supHeight=0.8;

	/** The current position of the painting cursor.
	 */
	protected int currentX=0;
	protected int currentY=0;

	//The position of the first bar drawn
	protected int currentXStart = 0;

	// code 39 Table
	/** code 39 table.
	 */
	protected String[][] set39={{"0","nnnwwnwnn"},{"1","wnnwnnnnw"},{"2","nnwwnnnnw"},{"3","wnwwnnnnn"},{"4","nnnwwnnnw"},{"5","wnnwwnnnn"},{"6","nnwwwnnnn"},{"7","nnnwnnwnw"},{"8","wnnwnnwnn"},{"9","nnwwnnwnn"},{"A","wnnnnwnnw"},{"B","nnwnnwnnw"},{"C","wnwnnwnnn"},{"D","nnnnwwnnw"},{"E","wnnnwwnnn"},{"F","nnwnwwnnn"},{"G","nnnnnwwnw"},{"H","wnnnnwwnn"},{"I","nnwnnwwnn"},{"J","nnnnwwwnn"},{"K","wnnnnnnww"},{"L","nnwnnnnww"},{"M","wnwnnnnwn"},{"N","nnnnwnnww"},{"O","wnnnwnnwn"},{"P","nnwnwnnwn"},{"Q","nnnnnnwww"},{"R","wnnnnnwwn"},{"S","nnwnnnwwn"},{"T","nnnnwnwwn"},{"U","wwnnnnnnw"},{"V","nwwnnnnnw"},{"W","wwwnnnnnn"},{"X","nwnnwnnnw"},{"Y","wwnnwnnnn"},{"Z","nwwnwnnnn"},{"-","nwnnnnwnw"},{".","wwnnnnwnn"},{" ","nwwnnnwnn"},{"$","nwnwnwnnn"},{"/","nwnwnnnwn"},{"+","nwnnnwnwn"},{"%","nnnwnwnwn"},{"*","nwnnwnwnn"}};
	/** code 2 of 5 table.
	 */
	protected String[][] set25={{"0","nnwwn"},{"1","wnnnw"},{"2","nwnnw"},{"3","wwnnn"},{"4","nnwnw"},{"5","wnwnn"},{"6","nwwnn"},{"7","nnnww"},{"8","wnnwn"},{"9","nwnwn"}};
	/** code MSI table.
	 */
	protected String[][] setMSI={{"0","nwnwnwnw"},{"1","nwnwnwwn"},{"2","nwnwwnnw"},{"3","nwnwwnwn"},{"4","nwwnnwnw"},{"5","nwwnnwwn"},{"6","nwwnwnnw"},{"7","nwwnwnwn"},{"8","wnnwnwnw"},{"9","wnnwnwwn"}};
	/** code 11 table.
	 */
	protected String[][] set11={{"0","nnnnw"},{"1","wnnnw"},{"2","nwnnw"},{"3","wwnnn"},{"4","nnwnw"},{"5","wnwnn"},{"6","nwwnn"},{"7","nnnww"},{"8","wnnwn"},{"9","wnnnn"},{"-","nnwnn"}};
	/** CODABAR table.
	 */
	protected String[][] setCODABAR={{"0","nnnnnww"},{"1","nnnnwwn"},{"2","nnnwnnw"},{"3","wwnnnnn"},{"4","nnwnnwn"},{"5","wnnnnwn"},{"6","nwnnnnw"},{"7","nwnnwnn"},{"8","nwwnnnn"},{"9","wnnwnnn"},{"-","nnnwwnn"},{"$","nnwwnnn"},{":","wnnnwnw"},{"/","wnwnnnw"},{".","wnwnwnn"},{"+","nnwnwnw"},{"A","nnwwnwn"},{"B","nwnwnnw"},{"C","nnnwnww"},{"D","nnnwwwn"}};
	/** code 93 table.
	 */
	protected String[][] set93={{"0","131112"},{"1","111213"},{"2","111312"},{"3","111411"},{"4","121113"},{"5","121212"},{"6","121311"},{"7","111114"},{"8","131211"},{"9","141111"},{"A","211113"},{"B","211212"},{"C","211311"},{"D","221112"},{"E","221211"},{"F","231111"},{"G","112113"},{"H","112212"},{"I","112311"},{"J","122112"},{"K","132111"},{"L","111123"},{"M","111222"},{"N","111321"},{"O","121122"},{"P","131121"},{"Q","212112"},{"R","212211"},{"S","211122"},{"T","211221"},{"U","221121"},{"V","222111"},{"W","112122"},{"X","112221"},{"Y","122121"},{"Z","123111"},{"-","121131"},{".","311112"},{" ","311211"},{"$","321111"},{"/","112131"},{"+","113121"},{"%","211131"},{"_1","121221"},{"_2","312111"},{"_3","311121"},{"_4","122211"}};
	/** code UPCA left table.
	 */
	protected String[][] setUPCALeft={{"0","3211"},{"1","2221"},{"2","2122"},{"3","1411"},{"4","1132"},{"5","1231"},{"6","1114"},{"7","1312"},{"8","1213"},{"9","3112"}};
		/** code UPCA right table.
	 */
	protected String[][] setUPCARight={{"0","3211"},{"1","2221"},{"2","2122"},{"3","1411"},{"4","1132"},{"5","1231"},{"6","1114"},{"7","1312"},{"8","1213"},{"9","3112"}};
	/** code UPCE odd table.
	 */

	protected String[][] setUPCEOdd={{"0","3211"},{"1","2221"},{"2","2122"},{"3","1411"},{"4","1132"},{"5","1231"},{"6","1114"},{"7","1312"},{"8","1213"},{"9","3112"}};
	/** code UPCE even table.
	 */
	protected String[][] setUPCEEven={{"0","1123"},{"1","1222"},{"2","2212"},{"3","1141"},{"4","2311"},{"5","1321"},{"6","4111"},{"7","2131"},{"8","3121"},{"9","2113"}};

	// encoding for extended code 39
	/** code 39 extended table.
	 */
	protected String[] set39Ext={"%U","$A","$B","$C","$D","$E","$F","$G","$H","$I","$J","$K","$L","$M","$N","$O","$P","$Q","$R","$S","$T","$U","$V","$W","$X","$Y","$Z","%A","%B","%C","%D","%E"," ","/A","/B","/C","/D","/E","/F","/G","/H","/I","/J","/K","/L","-",".","/O","0","1","2","3","4","5","6","7","8","9","/Z","%F","%G","%H","%I","%J","%V","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","%K","%L","%M","%N","%O","%W","+A","+B","+C","+D","+E","+F","+G","+H","+I","+J","+K","+L","+M","+N","+O","+P","+Q","+R","+S","+T","+U","+V","+W","+X","+Y","+Z","%P","%Q","%R","%S","%T"};
/** code 93 extended table.
	 */	
	protected String[] set93Ext={"_2U","_1A","_1B","_1C","_1D","_1E","_1F","_1G","_1H","_1I","_1J","_1K","_1L","_1M","_1N","_1O","_1P","_1Q","_1R","_1S","_1T","_1U","_1V","_1W","_1X","_1Y","_1Z","_2A","_2B","_2C","_2D","_2E"," ","_3A","_3B","_3C","_3D","_3E","_3F","_3G","_3H","_3I","_3J","_3K","_3L","-",".","_3O","0","1","2","3","4","5","6","7","8","9","_3Z","_2F","_2G","_2H","_2I","_2J","_2V","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","_2K","_2L","_2M","_2N","_2O","_2W","_4A","_4B","_4C","_4D","_4E","_4F","_4G","_4H","_4I","_4J","_4K","_4L","_4M","_4N","_4O","_4P","_4Q","_4R","_4S","_4T","_4U","_4V","_4W","_4X","_4Y","_4Z","_2P","_2Q","_2R","_2S","_2T"};
/** code UPCE system 0 table.
	 */
	protected String[] UPCESystem0={"EEEOOO","EEOEOO","EEOOEO","EEOOOE","EOEEOO","EOOEEO","EOOOEE","EOEOEO","EOEOOE","EOOEOE"};
/** code UPCE system 1 table.
	 */	protected String[] UPCESystem1={"OOOEEE","OOEOEE","OOEEOE","OOEEEO","OEOOEE","OEEOOE","OEEEOO","OEOEOE","OEOEEO","OEEOEO"};

/** code EAN left A table.
	 */		
	protected String[][] setEANLeftA={{"0","3211"},{"1","2221"},{"2","2122"},{"3","1411"},{"4","1132"},{"5","1231"},{"6","1114"},{"7","1312"},{"8","1213"},{"9","3112"}};
/** code EAN left B table.
	 */		
	protected String[][] setEANLeftB={{"0","1123"},{"1","1222"},{"2","2212"},{"3","1141"},{"4","2311"},{"5","1321"},{"6","4111"},{"7","2131"},{"8","3121"},{"9","2113"}};
/** code EAN right table.
	 */		
	protected String[][] setEANRight={{"0","3211"},{"1","2221"},{"2","2122"},{"3","1411"},{"4","1132"},{"5","1231"},{"6","1114"},{"7","1312"},{"8","1213"},{"9","3112"}};

/** code EAN  table.
	 */		
	protected String[] setEANCode={"AAAAA","ABABB","ABBAB","ABBBA","BAABB","BBAAB","BBBAA","BABAB","BABBA","BBABA"};

	/** 5 digit supplement table.
	 */	
	protected String[] fiveSuplement={"EEOOO","EOEOO","EOOEO","EOOOE","OEEOO","OOEEO","OOOEE","OEOEO","OEOOE","OOEOE"};
/** code 128 table.
	 */	
	protected String[] set128={"212222","222122","222221","121223","121322","131222","122213","122312","132212","221213","221312","231212","112232","122132","122231","113222","123122","123221","223211","221132","221231","213212","223112","312131","311222","321122","321221","312212","322112","322211","212123","212321","232121","111323","131123","131321","112313","132113","132311","211313","231113","231311","112133","112331","132131","113123","113321","133121","313121","211331","231131","213113","213311","213131","311123","311321","331121","312113","312311","332111","314111","221411","431111","111224","111422","121124","121421","141122","141221","112214","112412","122114","122411","142112","142211","241211","221114","413111","241112","134111","111242","121142","121241","114212","124112","124211","411212","421112","421211","212141","214121","412121","111143","111341","131141","114113","114311","411113","411311","113141","114131","311141","411131"};
/** code 128, character set A
	 */	
//	protected String[] set128A={" ","!","\"","#","$","%","&","'","(",")","*","+",",","-",".","/","0","1","2","3","4","5","6","7","8","9",":",";","<","=",">","?","@","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","[","\\","]","^","_",""+((char) 0),""+((char) 1),""+((char) 2),""+((char) 3),""+((char) 4),""+((char) 5),""+((char) 6),""+((char) 7),""+((char) 8),""+((char) 9),""+((char) 10),""+((char) 11),""+((char) 12),""+((char) 13),""+((char) 14),""+((char) 15),""+((char) 16),""+((char) 17),""+((char) 18),""+((char) 19),""+((char) 20),""+((char) 21),""+((char) 22),""+((char) 23),""+((char) 24),""+((char) 25),""+((char) 26),""+((char) 27),""+((char) 28),""+((char) 29),""+((char) 30),""+((char) 31),""+((char) 196),""+((char) 197),""+((char) 198),""+((char) 199),""+((char) 200),""+((char) 201),""+((char) 202)};
/** code 128, character set B
	 */	
	protected String[] set128B={" ","!","\"","#","$","%","&","'","(",")","*","+",",","-",".","/","0","1","2","3","4","5","6","7","8","9",":",";","<","=",">","?","@","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","[","\\","]","^","_","`","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z","{","|","}","~",""+((char) 195),""+((char) 196),""+((char) 197),""+((char) 198),""+((char) 199),""+((char) 200),""+((char) 201),""+((char) 202)};
/** code 128, character set C
	 */	
	protected String[] set128C={"00","01","02","03","04","05","06","07","08","09","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41","42","43","44","45","46","47","48","49","50","51","52","53","54","55","56","57","58","59","60","61","62","63","64","65","66","67","68","69","70","71","72","73","74","75","76","77","78","79","80","81","82","83","84","85","86","87","88","89","90","91","92","93","94","95","96","97","98","99",""+((char) 200)+((char) 200),""+((char) 201)+((char) 201),""+((char) 202)+((char) 202)};
/** code POSTNET table
	 */
	protected  String[][] setPOSTNET={{"0","11000"},{"1","00011"},{"2","00101"},{"3","00110"},{"4","01001"},{"5","01010"},{"6","01100"},{"7","10001"},{"8","10010"},{"9","10100"}};
/** code PLANET table
	 */
	protected  String[][] setPLANET= {{"0","00111"},{"1","11100"},{"2","11010"},{"3","11001"},{"4","10110"},{"5","10101"},{"6","10011"},{"7","01110"},{"8","01101"},{"9","01011"}};
	
	//Added by TB 6/2006, property for increasing the white space between bars 
	//for increased readability on low resolution printers.
	private float PropWhiteBarIncrease = 0;
 /** The percent of the XDimension to increase the white bars. */
	public float WhiteBarIncrease = 0;
/** Sets the WhiteBarIncrease */
	public void setWhiteBarIncrease(float newWhiteBarIncrease)
	{
		PropWhiteBarIncrease = newWhiteBarIncrease;
		//value must be between 1 and 99
		if (PropWhiteBarIncrease < 1 && PropWhiteBarIncrease != 0)
			PropWhiteBarIncrease = 1;

		if (PropWhiteBarIncrease > 99)
			PropWhiteBarIncrease = 99;

		//format the new value to decimal
		if(newWhiteBarIncrease >= 1 && newWhiteBarIncrease <= 99)
			newWhiteBarIncrease = newWhiteBarIncrease/100;

		WhiteBarIncrease = newWhiteBarIncrease;
		invalidate();
	}
/** Gets the WhiteBarIncrease */

	public float getWhiteBarIncrease()
	{
		return PropWhiteBarIncrease;
	}

	//Added by TB 6/2006, Propety for grouping human readable text for increased readability, default is 0 
	int ValSpliceText = 0;
/** Sets the number of digits to group together in the human readable text.  Valid options are 3,4,or 5.*/
	public void setCharacterGrouping (int newValSpliceText)
	{
		if (newValSpliceText >= 3 && newValSpliceText <=5 && barType!=ONECODE)
			ValSpliceText = newValSpliceText;
		else 
			ValSpliceText = 0;
		invalidate();
	}
/** Returns the number of characters that will be grouped together in the human readable text. */
	public int getCharacterGrouping ()
	{
		return ValSpliceText;
	}
	//added by TB 6/2006, Property for the thickness of vertical bearer bars. default is 0, which is disabled.
	//valid values are 0 - 10
	/** The value of the vertical bearer bar.  This value is a function of the XDimension.  Valid options are 0-10. */
	
	public double fBearerBarVert = 0;
	private double PropBearerBarVert = 0;
	/** Sets the value of the vertical bearer bar.  This value is a function of the XDimension.  Valid options are 0-10. */
	public void setBearerBarVertical (double newBearerBarVert)
	{
		
		if (newBearerBarVert < 0)
			newBearerBarVert = 0;
		if (newBearerBarVert > 10)
			newBearerBarVert = 10;
			
		PropBearerBarVert = newBearerBarVert;
		fBearerBarVert = newBearerBarVert;
		
		 if (barType ==POSTNET || barType ==PLANET || barType == ONECODE)
			PropBearerBarVert = 0;
		invalidate();
	}
	/** Gets the value of the vertical bearer bar.  This value is a function of the XDimension.  Valid options are 0-10. */

	public double getBearerBarVertical()
	{
		return PropBearerBarVert;
	}

	//added by TB 6/2006, Property for the thickness of horizontal bearer bars. default is 0, which is disabled.
	//valid values are 0 - 10
	/** The value of the horizontal bearer bar.  This value is a function of the XDimension.  Valid options are 0-10. */
	public double fBearerBarHori = 0;
	private double PropBearerBarHori = 0;
	/** Sets the value of the horizontal bearer bar.  This value is a function of the XDimension.  Valid options are 0-10. */

	public void setBearerBarHorizontal (double newBearerBarHori)
	{  
		     
		if (newBearerBarHori < 0)
			newBearerBarHori = 0;
		if (newBearerBarHori > 10)
			newBearerBarHori = 10;
			
		PropBearerBarHori = newBearerBarHori;
		fBearerBarHori = newBearerBarHori;
		
		if (barType ==POSTNET || barType ==PLANET || barType == ONECODE)
			PropBearerBarHori = 0;
			
		invalidate();
	}
	/** Gets the value of the horizontal bearer bar.  This value is a function of the XDimension.  Valid options are 0-10. */

	public double getBearerBarHorizontal()
	{
		return PropBearerBarHori;
	}

	/** adds a bar to the bar code at the currentX position.
	 */
	protected void addBar(Graphics g,int w,boolean black,int bartopmargin,boolean doPaint) {
		
		if ((black)&& (doPaint))		
		{			
			g.setColor(barColor);
			g.fillRect(currentX,+topMarginPixels+bartopmargin,w,barHeightPixels+extraHeight-bartopmargin);
		}
		// move pointer
		currentX=currentX+w;
		//Added by TB 6/2006, increase the white space between bars
		fXdimPixels = (float)XdimPixels;
		currentX = currentX +  (int)(fXdimPixels * WhiteBarIncrease);
	}
	
	/** paints a Postnet char.
	 */	
	protected void paintPostNetChar(Graphics g,String pattern)  {
		char cBar;
		//int shortBarPixels=(int) (postnetHeightTallBar*resolution);//original
		//int tallBarPixels= (int) (postnetHeightShortBar*resolution);//original
		int shortBarPixels=(int) (postnetHeightShortBar*resolution);
		int tallBarPixels= (int) (postnetHeightTallBar*resolution);
		g.setColor(barColor);
		for (int i=0;i<pattern.length();i++) {

			cBar=pattern.charAt(i);

			//if (cBar=='0') g.fillRect(currentX,+topMarginPixels,narrowBarPixels,tallBarPixels+extraHeight);//original
			//if (cBar=='1') g.fillRect(currentX,+topMarginPixels+(tallBarPixels-shortBarPixels),narrowBarPixels,shortBarPixels+extraHeight);//original
			if (cBar=='1') g.fillRect(currentX,+topMarginPixels,narrowBarPixels,tallBarPixels+extraHeight);
			if (cBar=='0') g.fillRect(currentX,+topMarginPixels+(tallBarPixels-shortBarPixels),narrowBarPixels,shortBarPixels+extraHeight);			
			// bar		
            		currentX=currentX+narrowBarPixels;
			// space
			currentX=currentX+widthBarPixels;
			fXdimPixels = (float)XdimPixels;
			currentX = currentX + (int)(fXdimPixels * WhiteBarIncrease);
		}
	}		

/** paints a Postnet code. */		
protected void paintPOSTNET(Graphics g, boolean doPaint) {

// Postnet codes have a trailing checksum that is the Modulo 10 of the sum of the digits. 
//	So to get the checksum, add all the digits in the code and subtract that sum from the 
//	next multiple of 10. The checksum for 32501 would be 9 (3+2+5+0+1=11; 20-11=9). 


	  int pos=0;
	  String c;
	  String c2;
	  int sum=0;
	  String codetmp=code;
	  String codeTextTmp = "";

	  // paint start char
	  paintPostNetChar(g,"1");

	  // calculate checksum
	  for (int i=(code.length()-1);i>=0;i--) {
		  c=""+code.charAt(i);
		  if (findChar(setPOSTNET,c)!= -1)
		  	sum=sum+findChar(setPOSTNET,c);
	  }

	  int checksum=(int) mod(sum,10);
	  if (checksum!=0) checksum=10-checksum;

	  // add checksum char
	  if (checkCharacter) codetmp=codetmp+(new Integer(checksum).toString());

	  // paint char in string
	  for (int i=0;i<codetmp.length();i++) {
		  c=""+codetmp.charAt(i);
		  	pos=findChar(setPOSTNET,c);
		  	if (pos != -1)
		  	{
		  		paintPostNetChar(g,setPOSTNET[pos][1]);
		  		codeTextTmp = codeTextTmp +(new Integer(pos).toString());
		  	}
	  }


	  // paint stop char
	  paintPostNetChar(g,"1");
	  
		if (checkCharacterInText || (!checkCharacter && !checkCharacterInText))
			codeText=codeTextTmp;
	  else
	  	codeText = codeTextTmp.substring(0,codeTextTmp.length() - 1);
	}	
	 

	/** find a char in a charset table
	 */
	protected int findChar(String[][] table,String c) {

		for (int i=0;i<table.length;i++) {
			if (c.compareTo(table[i][0])==0) return i;
		}

		return -1;
	}

/** paints a PLANET code. */		
protected void paintPLANET(Graphics g, boolean doPaint) {

// PLANET codes have a trailing checksum that is the Modulo 10 of the sum of the digits. 
//	So to get the checksum, add all the digits in the code and subtract that sum from the 
//	next multiple of 10. The checksum for 32501 would be 9 (3+2+5+0+1=11; 20-11=9). 


	  int pos=0;
	  String c;
	  String c2;
	  int sum=0;
	  String codetmp=code;
		String codeTextTmp = "";
		
	  // paint start char
	  paintPostNetChar(g,"1");

	  // calculate checksum
	  for (int i=(code.length()-1);i>=0;i--) {
		  c=""+code.charAt(i);
		  if (findChar(setPLANET,c) != -1)
		  	sum=sum+findChar(setPLANET,c);
	  }

	  int checksum=(int) mod(sum,10);
	  if (checksum!=0) checksum=10-checksum;

	  // add checksum char
	  if (checkCharacter) codetmp=codetmp+(new Integer(checksum).toString());

	  // paint char in string
	  for (int i=0;i<codetmp.length();i++) {
		  c=""+codetmp.charAt(i);
		  pos=findChar(setPLANET,c);
		  if (pos != -1)
		  {
		  	paintPostNetChar(g,setPLANET[pos][1]);
		  	codeTextTmp = codeTextTmp +(new Integer(pos).toString());
		  }
	  }

	  // paint stop char
	  paintPostNetChar(g,"1");
	  
	  if (checkCharacterInText || (!checkCharacter && !checkCharacterInText))
			codeText=codeTextTmp;
	  else
	  	codeText = codeTextTmp.substring(0,codeTextTmp.length() - 1);
	}	


/** paints an interleaved 2 of 5 code.	*/	
	protected void paintInterleaved25(Graphics g, boolean doPaint) {

// Interleaved 2 of 5 code is a numeric only bar code.
// Each character of this code is represented by five elements, two
// wide and three narrow. Wide elements are decoded as binary one (1),
// and narrow elements are decoded as binary zero (0). The wide to narrow
// element ratio should be between two and three.
// Whether or not the elements used to encode a character are bars or
// spaces depends upon the location
// of the character within the message. The first character of the message is encoded
// into the bars immediately following the start character. The second character
// of the message is encoded into the spaces between the bars of the first character,
// thus eliminating the intercharacter space. Because of this, Interleaved 2 of 5 is
// a continuous bar code.

// Due to the interleaving of the characters, the number of characters in an
// Interleaved 2 of 5 message must be even. The check character, if used, must
// be included in the character count. If the message has an odd number of characters, add a leading zero (0) to the message.

// Interleaved 2 of 5 has an optional modulus 10 check character, which is printed at the end of the message. The value of the check character is determined by the following six step procedure.
// Identify even and odd positioned characters in the message with the rightmost data character always defined as an even positioned character.
// - Sum the numeric values of the odd positioned characters.
// - Sum the numeric values of the even positioned characters and multiply this total by three.
// - Sum the odd and even totals from steps two and three.
// - Determine the smallest number which, when added to the sum in step four, will result in a multiple of ten. This number is the value of the check character.
// - Determine if the number of characters (message plus check character) is even or odd. If it is odd, add a leading, nonsignificant zero to the message to produce an even number of characters.

	  int pos=0;
	  int pos2=0;
	  String c;
	  String c2;
	  int sum=0;
	  String codetmp=code;
	  

		    //Quick check to test for improper character. //brj 9/14/06
		    	for (int i=0;i<code.length();i++) 
			{
				if (((int)code.charAt(i)< 48) || ((int)code.charAt(i) > 57) )
				
				{
					codeText="Incorrect Data To Encode";	
					return;
				}
			}			

	  // paint start char
	  paintChar(g,"bwbw","nnnn", doPaint);

	  // leading 0 needed?
	  String l="";
	  if ((mod(code.length(),2)==0) && (checkCharacter)) codetmp="0"+code;
	  if ((mod(code.length(),2)==1) && (!checkCharacter)) codetmp="0"+code;

	  // calculate checksum
	  int sumeven=0;
	  int sumodd=0;
	  boolean even=true;
	  for (int i=(codetmp.length()-1);i>=0;i--) {
		  c=""+codetmp.charAt(i);
		  if (even) sumeven=sumeven+findChar(set25,c);
		  else sumodd=sumodd+findChar(set25,c);
		  even=(!even);
	  }

	  int checksum=(sumeven*3)+sumodd;
	  checksum=(int) mod(checksum,10);
	  if (checksum!=0) checksum=10-checksum;

	  // 48 is the asc value of the char 0
	  if (checkCharacter) codetmp=codetmp+(new Integer(checksum).toString());

	  // paint char in string
	  for (int i=0;i<codetmp.length();i=i+2) {
		  c=""+codetmp.charAt(i);
		  c2=""+codetmp.charAt(i+1);
		  pos=findChar(set25,c);
		  pos2=findChar(set25,c2);

		  // draw bar of first char and space of second char, all 5 components per char
		  for (int j=0;j<5;j++) {
			   paintChar(g,"b",""+set25[pos][1].charAt(j), doPaint);
			   paintChar(g,"w",""+set25[pos2][1].charAt(j), doPaint);
		  }
	  }


	  // paint stop char
	  paintChar(g,"bwb","wnn", doPaint);

	  if (checkCharacterInText) codeText=codetmp;
	  else codeText=code;
	}

		/** paints an industrial 2 of 5 code.
	 */	
protected void paintIND25(Graphics g, boolean doPaint) {

// The value of the checksum character is determined by the following six step
//procedure:

// 1. Identify even and odd positioned characters in the message with the
//    right-hand message character ALWAYS defined as an even positioned character.
// 2. Sum the numeric values of the odd positioned characters.
// 3. Sum the numeric values of the even positioned characters and multiply the total
//    value by three.
// 4. Sum the odd and even totals from steps 2 and 3.
// 5. Determine the smallest number which, when added to the sum in step 4, will result
//    in a multiple of 10. This number is the value of the checksum character.



	  int pos=0;
	  String c;
	  String c2;
	  int sum=0;
	  String codetmp=code;

	  // paint start char
	  paintChar(g,"bwbwbw","wwwwnw", doPaint);

/*
	  // calculate checksum
	  int sumeven=0;
	  int sumodd=0;
	  boolean even=true;
	  for (int i=(codetmp.length()-1);i>=0;i--) {
		  c=""+codetmp.charAt(i);
		  if (even) sumeven=sumeven+findChar(set25,c);
		  else sumodd=sumodd+findChar(set25,c);
		  even=(!even);
	  }

	  int checksum=(sumeven*3)+sumodd;
	  checksum=(int) mod(checksum,10);
	  if (checksum!=0) checksum=10-checksum;

	  // 48 is the asc value of the char 0
	  if (checkCharacter) codetmp=codetmp+(new Integer(checksum).toString());
*/
	  // paint char in string
	  for (int i=0;i<codetmp.length();i++) {
		  c=""+codetmp.charAt(i);
		  pos=findChar(set25,c);
		  if (pos>=0) {
			  for (int j=0; j<set25[pos][1].length();j++) {
			   paintChar(g,"b",""+set25[pos][1].charAt(j), doPaint); // bar
			   paintChar(g,"w","w", doPaint); // narrow space
			  }
		  }
	  }

	  // paint stop char
	  paintChar(g,"bwbwb","wwnww", doPaint);
	}


	/** caculates the UPC/EAN check character.
	 */	
protected String UPCEANCheck(String s) {

// The check character for the UPC/EAN codes is generated by the following method:

// 1. Designate the rightmost character odd.
// 2. Sum all of the characters in the odd positions and multiply the result by three.
// 3. Sum all of the characters in the even positions
// 4. Add the odd and even totals from steps two and three.
// 5. Determine the smallest number that when added to the result from step four,
// will result in a multiple of 10. This is the check character.

	boolean odd=true;
	int sumodd=0;
	int sum=0;
	int c=0;

	for (int i=(s.length()-1);i>=0;i--) {


		 if (odd) sumodd=sumodd+(new Integer(""+s.charAt(i)).intValue());
		 else sum=sum+(new Integer(""+s.charAt(i)).intValue());
		 odd=(!odd);
	  
	}

	sum=(sumodd*3)+sum;

	c=(int) mod(sum,10);

	if (c!=0) c=10-c;

	return ""+c;
}



/** paints a UPCA code.	 */	

protected void paintUPCA(Graphics g, boolean doPaint) {

// The UPC-A symbol is physically constructed as follows:
//
// Left hand guard bars, encoded 101
// Number system character, encoded as a left hand character
// First five data characters, encoded as left hand characters
// Center guard pattern, encoded 01010
// Last five data characters, encoded as right hand characters
// Check character, encoded as a right hand character
// Right hand guard bars, encoded 101

	  int pos=0;
	  String c;
	  String c2;
	  int sum=0;
	  String CodeTmp = "";//tb 7/26/2006
	
	  
	  
	      //Quick check to test for improper character. //brj 9/14/06
		    	for (int i=0;i<code.length();i++) 
			{
				if (((int)code.charAt(i)< 48) || ((int)code.charAt(i) > 57) )
				
				{
					codeText="Incorrect Data To Encode";	
					return;
				}
			}
	  
	  
	  
	  
	  if (code.length()<11) return;
	  
	//check for add-ons and if they exist set them up in the barcode
	if(code.length()==13) {
		supplement=code.substring(11,13);
		UPCEANSupplement2=true;
		UPCEANSupplement5=false;//tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==14) {
		supplement=code.substring(12,14);
		UPCEANSupplement2=true;
		UPCEANSupplement5=false;//tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==16) {
		supplement=code.substring(11,16);
		UPCEANSupplement5=true;
		UPCEANSupplement2=false;//tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==17) {
		supplement=code.substring(12,17);
		UPCEANSupplement5=true;
		UPCEANSupplement2=false;//tb 7/26/2006
		//code=code.substring(0,11);
	}

	//add in the check digit
      //this code fixes a bug that would auto-increment the check digit in the applet
	CodeTmp=code.substring(0,11);
      CodeTmp=CodeTmp + UPCEANCheck(CodeTmp);

	  // paint start char
	  paintGuardChar(g,"bwb","nnn",0,doPaint);

	  leftGuardBar=currentX;

	  // paint char in string
	  for (int i=0;i<CodeTmp.length();i++) {
		  c=""+CodeTmp.charAt(i);

		  pos=-1;
		  if (i<=5) {
			pos=findChar(setUPCALeft,c);
			paintChar(g,"wbwb",setUPCALeft[pos][1],doPaint);
		  }
		  else {
			pos=findChar(setUPCARight,c);
			paintChar(g,"bwbw",setUPCARight[pos][1],doPaint);
		  }

		  // center guard Pattern
		  if (i==5)  {

			   centerGuardBarStart=currentX;

			   paintGuardChar(g,"wbwbw","nnnnn",0,doPaint);

			   centerGuardBarEnd=currentX;
		  }
	  }

	  rightGuardBar=currentX;

	  // paint stop char
	  paintGuardChar(g,"bwb","nnn",0, doPaint);
	  
	  endOfCode=currentX;
	  
	  codeText = CodeTmp; //tb 27/26/2006
	   
	  // supplement
	  if (UPCEANSupplement2) 
	  	paintSup2(g,supplement, doPaint);
	  else if (UPCEANSupplement5) paintSup5(g,supplement, doPaint);	
	}


	/** paints a EAN13 code.
	 */	
protected void paintEAN13(Graphics g, boolean doPaint) {


//	The physical structure of the EAN-13 is as follows:

// - Left hand guard bars, encoded 101
// - Second flag character, encoded as in the following tables
// - First five data characters, encoded as in the following tables
// - Center guard pattern, encoded 01010
// - Last five data characters, encoded as right hand characters
// - Check character, encoded as a right hand character
// - Right hand guard bars, encoded 101
// The thirteenth digit is encoded in the pattern of parity in the second flag character and the
// first five data characters. The left hand A and the left hand B character sets are used.



	  int pos=0;
	  String c;
	  String c2;
	  int sum=0;
	  String[][] leftset;
		String CodeTmp = "";//tb 7/26/2006
		
		
		    //Quick check to test for improper character. //brj 9/14/06
		    	for (int i=0;i<code.length();i++) 
			{
				if (((int)code.charAt(i)< 48) || ((int)code.charAt(i) > 57) )
				
				{
					codeText="Incorrect Data To Encode";	
					return;
				}
			}
		
		
		
      if (code.length()<12) return;

	//check for add-ons and if they exist set them up in the barcode
	if(code.length()==14) {
		supplement=code.substring(12,14);
		UPCEANSupplement2=true;
		UPCEANSupplement5=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==15) {
		supplement=code.substring(13,15);
		UPCEANSupplement2=true;
		UPCEANSupplement5=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==17) {
		supplement=code.substring(12,17);
		UPCEANSupplement5=true;
		UPCEANSupplement2=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==18) {
		supplement=code.substring(13,18);
		UPCEANSupplement5=true;
		UPCEANSupplement2=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}

	//add in the check digit
      //this code fixes a bug that would auto-increment the check digit in the applet
	CodeTmp=code.substring(0,12);
      CodeTmp=CodeTmp + UPCEANCheck(CodeTmp);

	  // paint start char
	  paintGuardChar(g,"bwb","nnn",0,doPaint);
	  leftGuardBar=currentX;

	 // second flag char
	  String sets=setEANCode[new Integer(""+CodeTmp.charAt(0)).intValue()];
        pos=findChar(setEANLeftA,""+CodeTmp.charAt(1));
	  paintChar(g,"wbwb",setEANLeftA[pos][1],doPaint);

	  // paint char in string
	  for (int i=2;i<12;i++) {
		  c=""+CodeTmp.charAt(i);

		  pos=-1;
		  if (i<=6) {

			// use set A or B?
			leftset=setEANLeftA;
			if (sets.charAt(i-2)=='B') leftset=setEANLeftB;

			pos=findChar(leftset,c);
			paintChar(g,"wbwb",leftset[pos][1],doPaint);
		  }
		  else {
			pos=findChar(setEANRight,c);
			paintChar(g,"bwbw",setEANRight[pos][1],doPaint);
		  }

		  // center guard Pattern
		  if (i==6)  {
			   centerGuardBarStart=currentX;
			   paintGuardChar(g,"wbwbw","nnnnn",0,doPaint);
			   centerGuardBarEnd=currentX;
		  }

	  }

	  // check character
      pos=findChar(setEANRight,""+CodeTmp.charAt(12));
	  paintChar(g,"bwbw",setEANRight[pos][1],doPaint);

	  rightGuardBar=currentX;

	  // paint stop char
	  paintGuardChar(g,"bwb","nnn",0,doPaint);
	  
	  codeText = CodeTmp; //tb 27/26/2006
	  
	  endOfCode=currentX;
	  
	  // supplement **
	  // supplement **
	  if (UPCEANSupplement2) paintSup2(g,supplement, doPaint);
	  else if (UPCEANSupplement5) paintSup5(g,supplement, doPaint);		  

	}



private int findInArray(String[] s1, String c) {

  for (int j=0;j<s1.length;j++)
	  if (s1[j].compareTo(c)==0) return j;
  return -1;
}

//this will only check for digits 0-9 only
protected boolean isDigit(char n) {  //brj
if ((char)n >=48 && (char)n <=57)
		return true;
else
		return false;
	}

		
/** paints a code 128 barcode from character set A, B or C and encode ASCII 0 to 31 in the barcode if it exists. */	
protected void paintCode128(Graphics g, boolean doPaint) {

// Code 128 is a continuous, multilevel, full ASCII code. Each of the Code 128
//	characters consists of three bars and three spaces. The bars and spaces may be
//	one, two, three, or four modules wide. The total length of each code 128 character
//	is eleven modules, with the total length of the bar modules odd, and the total
//	length of the space modules even.

// The character set consists of 103 different characters, three different
//	start characters, and one unique stop character


// A check character is mandatory for Code 128. The check character is a modulus
//	103 sum of the character values, weighted by position in the message. The check
//	character is calculated by summing the value for the start character with the
//	products from the value of the message characters times their position in the
//	message, with the character that follows the start character in the first position.
//	The stop character is not included in the calculation. The result is divided by 103,
//	and the character that corresponds to the remainder is the check character.

	  int pos=0;
	  int check=0;
	  String c;
	  String[] set;
	  String cChar;
	  String codetmp=code;
	  codeText=code;
	  int CurrentChar = 0;
	  int CorrectFNC = 0;
	  int CurrentCharNum = 0;
	  int dataLength = code.length();
	  
		
	if (processTilde)
	{
		codetmp="";
		int WeightValue = 0;
		int strBegin = 0;
		int strEnd =0;
		String CheckDigitValue = "";

		for(int i = 1; i <= dataLength; i++ )
		{
			if ( (i < dataLength - 2) && (code.charAt(i-1) == '~') && (code.charAt(i) == 'm') )
			{
				WeightValue = ((((int)code.charAt(i+1))-48)*10) + (((int)code.charAt(i+2))-48);
				if ((i - WeightValue) < 1) 
				{
					WeightValue = i-1;
				}
				strBegin = (i-1) - WeightValue;
				strEnd = WeightValue + strBegin;
					String tempStr = "";
				
				for (int j=0; j< dataLength -4; j++) {
					if ((int) code.charAt(j) ==126 && j < dataLength -3 && (Character.isDigit(code.charAt(j+1)) &&Character.isDigit(code.charAt(j+2))&& Character.isDigit(code.charAt(j+3))))
					{
							j = j+3;
						  continue;
					}
					else
					   tempStr+= code.charAt(j);
				}
				//String MODString=code.substring(strBegin,strEnd);
				String MODString = tempStr;
				CheckDigitValue=UPCEANCheck(MODString);
				codetmp = codetmp + CheckDigitValue;
				i = i + 3;
			}

			else if ( (i < dataLength - 2) && (code.charAt(i-1) == '~') && ((int)code.charAt(i)> 47) && ((int)code.charAt(i) < 58) )
			{
				CurrentCharNum = ((((int)code.charAt(i))-48)*100) + ((((int)code.charAt(i+1))-48)*10)  + (((int)code.charAt(i+2))-48);
				if ((CurrentCharNum > -1) && (CurrentCharNum < 255))
				{
					codetmp = codetmp + (char)(CurrentCharNum);
					i = i + 3;
				}
				else
				{
					codetmp = codetmp + code.charAt(i-1);
				}
			}
			
			else
			{
				codetmp = codetmp + code.charAt(i-1);
			}
		}
		CurrentCharNum = 0;
		code=codetmp;
		codetmp="";
		dataLength = code.length();
	}
	if (barType == UCC128)
			{//do nothing if  FNC1 is the first character
					if ( (int)code.charAt(0) == 202 || ((int)code.charAt(0) >= 212 && (int)code.charAt(0) <= 217)) 
						{
						}
					else//insert FNC1
						code = ((char) 202) + code;
						//codetmp = ((char) 202) + code;
			}
			
			dataLength = code.length();
			
          //Set defaults to codeset A
	  set=set128B;
	  int sum=103;

          //Perform conversions to encode ASCII 0 to 31 in the barcode
	  if (Code128Set!='0') {
             codetmp="";
             codeText="";
             for(int i = 1; i <= dataLength; i++ )  {
               //Get ASCII value of each character
               CurrentChar = (int)code.charAt(i-1);
               if((CurrentChar < 32) && (CurrentChar >= 0)) {
                  if (Code128Set=='A') codetmp = codetmp + (char)(CurrentChar + 96);
                  //For code set B, switch to code A, process function & switch back
                  if (Code128Set=='B') {
                    if (((int)code.charAt(i))<32) {
                    codetmp = codetmp + (char)201 + (char)(CurrentChar + 96) + (char)((int)code.charAt(i) + 96) + (char)200;
                    i++;
                    }
                    else codetmp = codetmp + (char)201 + (char)(CurrentChar + 96) + (char)200;
                  }
                  //For code set C, switch to code A, process function & switch back
                  if (Code128Set=='C') {
                    if (((int)code.charAt(i))<32) {
                    codetmp = codetmp + (char)201 + (char)201 + (CurrentChar + 64) + ((int)code.charAt(i) + 64) + "99";
                    i++;
                    }
                    else codetmp = codetmp + (char)201 + (char)201 + (CurrentChar + 64) + "99";
                  }
                  //For tabs and returns in text, show only spaces, hide all others from text
                  if((CurrentChar == 13)||(CurrentChar == 9)) codeText = codeText + "  ";
                  }
               else {
                  codeText = codeText + (char)CurrentChar;
                  codetmp = codetmp + (char)CurrentChar;
               }
              }
            }


	  if (Code128Set=='0') {
             set=set128B;
             codeText="";
             codetmp="";
             char C128Start = (char)204;
             char CurrentEncoding = 'B';
             // Here we select character set A, B or C for the START character
             CurrentCharNum = (int)code.charAt(0);
             if (CurrentCharNum < 32) C128Start = (char)203;
             if (CurrentCharNum > 31 && CurrentCharNum < 127) C128Start = (char)204;
             if ((dataLength > 3) && (CurrentCharNum>47 && CurrentCharNum<58) &&((int)code.charAt(1)>47 &&(int)code.charAt(1)<58) && ((int)code.charAt(2)>47 && (int)code.charAt(2)<58) && ((int)code.charAt(3)>47 && (int)code.charAt(3)<58)) C128Start = (char)205;
		     // BDA ADDED V5.04 FOR FNC2 SUPPORT
		     if (CurrentCharNum == 197) C128Start = (char)204;
		     // 202 is the FNC1, also check for ASCII 121 to 215
             if ( (CurrentCharNum == 202) || (CurrentCharNum > 211) ) C128Start = (char)205;
             
			 if (C128Start == (char)203) {
                CurrentEncoding = 'A';
                sum=103;
                paintChar(g,"bwbwbw","211412",doPaint);
             }
             if (C128Start == (char)204) {
                CurrentEncoding = 'B';
                sum=104;
                paintChar(g,"bwbwbw","211214", doPaint);
             }
             if (C128Start == (char)205) {
                CurrentEncoding = 'C';
                sum=105;
                paintChar(g,"bwbwbw","211232", doPaint);
             }
             for(int i = 1; i <= dataLength; i++ )  {
                // Get ASCII value of each character
                CurrentCharNum = (int)code.charAt(i-1);

                // Check for FNC1
                if ((i < dataLength-1) && ((CurrentCharNum == 202) || (CurrentCharNum > 211)) ) {
                      //Here we encode ASCII 202, the FNC1 into the codetmp
                      codetmp= codetmp + (char)202;
                }
				 
				 // BDA ADDED V5.04 FOR FNC2 SUPPORT
				 // Check for FNC2
				else if (CurrentCharNum == 197)
				{
					 //Here we encode ASCII 197, the FNC2 into the codetmp
					 if (CurrentEncoding == 'C') 
					 {
						 codetmp= codetmp + (char)200;
						 CurrentEncoding = 'B';
					 }
					 codetmp= codetmp + (char)197;
					 
				 }
				
					
				// Check for switching to character set C
                else if (((i <= dataLength - 3) && ((CurrentCharNum>47) && (CurrentCharNum<58) && ((int)code.charAt(i)>47) && ((int)code.charAt(i)<58) && ((int)code.charAt(i+1)>47) && ((int)code.charAt(i+1)<58) && ((int)code.charAt(i+2)>47) && ((int)code.charAt(i+2)<58)))
                   || ((i <= dataLength - 1) && (CurrentCharNum>47) && (CurrentCharNum<58) && ((int)code.charAt(i)>47) && ((int)code.charAt(i)<58) && (CurrentEncoding == 'C'))) 
                   
             
                   
                   
                   
               //   else if 	( (dataLength > 3 && i <= (dataLength - 2)) &&
		//	((((int) code.charAt(i)) && ((int) code.charAt(i + 1)) && 
			//((int) code.charAt(i + 2)) && (int) code.charAt(i + 3) ) || 
			//(i <= dataLength - 1 && ((int) code.charAt(i)) && 
			//((int) code.charAt(i + 1)) && CurrentEncoding == 'C' )))
                   
                   	{
         

	//////////////////////////////brj/////5/17/06////////////////////////////////////////////////////////////////////
         //brj 
             //check to see if we have an odd number of numbers to encode,
             //if so stay in current set for 1 number and then switch to save space
	 if (CurrentEncoding !='C')
	    
				{
							
      	      int j = i;
		  	 			int Factor = 3;  //

             			while (j <= dataLength  && isDigit((code.charAt(j-1))))
			
										{
											Factor = (4 - Factor);
											j++;
					 					}
	
            	if (Factor == 1)  
										{
											//if so stay in current set for 1 character to save space
											codetmp = codetmp + code.charAt(i-1); 
											  
											i++;
                     }

							Factor = 0;
		  }
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

                   
                   
                   // Switch to set C if not already in it
                   if (CurrentEncoding != 'C') {
                      codetmp= codetmp + (char)199;
                      CurrentEncoding = 'C';
                   }
                   
              

                   // Get the value of each number pair (ex: 5 and 6 = 5*10+6 =56)
                   CurrentCharNum = ((((int)code.charAt(i-1))-48)*10) + (((int)code.charAt(i))-48);
                   if((CurrentCharNum < 95) && (CurrentCharNum  >= 0))
                      codetmp= codetmp + (char)(CurrentCharNum + 32);
                   else if(CurrentCharNum > 94)
                      codetmp= codetmp + (char)(CurrentCharNum + 100);
                   i = i + 1;
                }
                   // Check for switching to character set A           
                else if ((i <= dataLength) && ((CurrentCharNum < 32) || ((CurrentEncoding == 'A') && (CurrentCharNum < 96)))) {
                   // Switch to set A if not already in it
                   if (CurrentEncoding != 'A') {
                      codetmp= codetmp + (char)201;
                      CurrentEncoding = 'A';
                   }
                   if (CurrentCharNum < 32) {
                      codetmp= codetmp + (char)(CurrentCharNum + 96);
                   }
                   else if (CurrentCharNum > 31) {
                      codetmp= codetmp + (char)CurrentCharNum;
                   }
                }
                   // Check for switching to character set B
                else if ((i <= dataLength) && (CurrentCharNum > 31 && CurrentCharNum < 127)) {
                   // Switch to set B if not already in it
                   if (CurrentEncoding != 'B') {
                      codetmp= codetmp + (char)200;
                      CurrentEncoding = 'B';
                   }
                   codetmp= codetmp + (char)CurrentCharNum;
                }
             }

          // FORMAT TEXT FOR AIs
          for(int i = 1; i <= dataLength; i++ )  {
             // Get ASCII value of each character
             CurrentCharNum = (int)code.charAt(i-1);
             // Check for FNC1
             CorrectFNC = 0;
             if ( (i < dataLength-1) && ( (CurrentCharNum == 202) || (CurrentCharNum > 211) ) ) 
             {
                // It appears that there is an AI
                // Get the value of each number pair (ex: 5 and 6 = 5*10+6 =56)
                CurrentCharNum = ((((int)code.charAt(i))-48)*10) + (((int)code.charAt(i+1))-48);


                // Is 2 digit AI by entering ASCII 212?
                if ((int)code.charAt(i-1) == 212)  {
                      codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + ") ";
                      i+=2;
                      CorrectFNC=1;
                }

                // Is 3 digit AI by entering ASCII 213?
                else if ( (i < dataLength-2) && ((int)code.charAt(i-1) == 213) ) {
                      codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + ") ";
                      i+=3;
                      CorrectFNC=1;
                }

                // Is 4 digit AI by entering ASCII 214?
                else if ( (i < dataLength-3) && ((int)code.charAt(i-1) == 214) ) {
                      codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + ") ";
                      i+=4;
                      CorrectFNC=1;
                }

                // Is 5 digit AI by entering ASCII 215?
                else if ( (i < dataLength-4) &&  ((int)code.charAt(i-1) == 215) ) {
                      codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + code.charAt(i+4) + ") ";
                      i+=5;
                      CorrectFNC=1;
                }

					// Is 6 digit AI by entering ASCII 216?
				else if ( (i < dataLength-5) &&  ((int)code.charAt(i-1) == 216) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + code.charAt(i+4) + code.charAt(i+5) + ") ";
					i+=6;
					CorrectFNC=1;
				}

					// Is 7 digit AI by entering ASCII 217?
				else if ( (i < dataLength-6) &&  ((int)code.charAt(i-1) == 217) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + code.charAt(i+4) + + code.charAt(i+5) + code.charAt(i+6) + ") ";
					i+=7;
					CorrectFNC=1;
				}


                // Is 2 digit AI auto-detected?
                else if ( (CurrentCharNum <= 30 && CurrentCharNum  >= 0) || (CurrentCharNum <=99 && CurrentCharNum  >= 90) ) {
                      codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + ") ";
                      i+=2;
                      CorrectFNC=1;
                }

                // Is 3 digit AI auto-detected?
                else if ( (i < dataLength-2) && ( (CurrentCharNum <= 49 && CurrentCharNum  >= 40) || (CurrentCharNum <=25 && CurrentCharNum  >= 23) ) ) {
                      codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + ") ";
                      i+=3;
                      CorrectFNC=1;
                }

                // Is 4 digit AI auto-detected?
                else if ( (i < dataLength-3) && ( (CurrentCharNum <= 81 && CurrentCharNum  >= 80) || (CurrentCharNum <=34 && CurrentCharNum  >= 31) ) ) {
                      codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + ") ";
                      i+=4;
                      CorrectFNC=1;
                }

                // Default to 4 digit AI?
                else if ( (i < dataLength-3) &&  (CorrectFNC==0) ) {
                      codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + ") ";
                      i+=4;
                }

             }
             else if ((int)code.charAt(i-1) < 32) codeText = codeText + " ";
			 else if ((int)code.charAt(i-1) == 197) codeText = codeText + " ";
             else if ( ((int)code.charAt(i-1) > 31) && ((int)code.charAt(i-1) < 128) ) codeText = codeText + code.charAt(i-1);
          }
          }

	  if (Code128Set=='B') {
             set=set128B;
             sum=104;
             }
	  if (Code128Set=='C') {
             set=set128C;
             sum=105;
             // Check for an even number of digits, add 0 if not even
             if((codetmp.length() % 2) == 1) {
             		
                codetmp = "0"+codetmp;
                codeText=codetmp;
                }
             }

	  // paint start char
	  if (Code128Set=='B') paintChar(g,"bwbwbw","211214",doPaint);
	  if (Code128Set=='C') paintChar(g,"bwbwbw","211232", doPaint);
    if (Code128Set=='A') paintChar(g,"bwbwbw","211412", doPaint);

	  // paint char in string
	  int w=1; // weight for checksum
	  for (int i=0;i<codetmp.length();i++) {
		  c=""+codetmp.charAt(i);

		  if (Code128Set=='C') {
			  // charater set C encodes two characters at a time
                          // 1st character
			  cChar=""+c;
			  i++;
                          // 2nd character added to the end of cChar
			  if (i<codetmp.length()) cChar=cChar+codetmp.charAt(i);

			  pos=findInArray(set128C,cChar);
			  if (pos>=0)  {paintChar(g,"bwbwbw",set128[pos],doPaint);
			               sum=sum+(pos*w);}
		  }
		  
		  else {
			// set A or B
		        pos=findInArray(set,c);
			if (pos>=0)  {paintChar(g,"bwbwbw",set128[pos],doPaint);
			         	sum=sum+(pos*w);}
		  }
		  w++;
	  }


	  if (checkCharacter) {
	    check=(int) mod(sum,103);
	    // check char
	    paintChar(g,"bwbwbw",set128[check],doPaint);
	  }

	  // paint stop char
	  paintChar(g,"bwbwbwb","2331112", doPaint);

}


/** paints a Code 128 Code for EAN-128 and UCC-128 specifications which is a mandatory character set C start character and the FNC1 code. */
protected void paintUCC128(Graphics g, boolean doPaint) {
	//Set defaults for UCC/EAN 128
	setCode128Set("AUTO");

	// Code 128 is a continuous, multilevel, full ASCII code. Each of the Code 128
	//	characters consists of three bars and three spaces. The bars and spaces may be
	//	one, two, three, or four modules wide. The total length of each code 128 character
	//	is eleven modules, with the total length of the bar modules odd, and the total
	//	length of the space modules even.

	// The character set consists of 103 different characters, three different
	//	start characters, and one unique stop character


	// A check character is mandatory for Code 128. The check character is a modulus
	//	103 sum of the character values, weighted by position in the message. The check
	//	character is calculated by summing the value for the start character with the
	//	products from the value of the message characters times their position in the
	//	message, with the character that follows the start character in the first position.
	//	The stop character is not included in the calculation. The result is divided by 103,
	//	and the character that corresponds to the remainder is the check character.

	int pos=0;
	int check=0;
	String c;
	String[] set;
	String cChar;
	String codetmp=code;
	codeText=code;
	int CurrentChar = 0;
	int CorrectFNC = 0;
	int CurrentCharNum = 0;
	int dataLength = code.length();

	if (processTilde)
	{
		codetmp="";
		int WeightValue = 0;
		int strBegin = 0;
		int strEnd =0;
		String CheckDigitValue = "";

		for(int i = 1; i <= dataLength; i++ )
		{
			if ( (i < dataLength - 2) && (code.charAt(i-1) == '~') && (code.charAt(i) == 'm') )
			{
				WeightValue = ((((int)code.charAt(i+1))-48)*10) + (((int)code.charAt(i+2))-48);
				if ((i - WeightValue) < 1) 
				{
					WeightValue = i-1;
				}
				strBegin = (i-1) - WeightValue;
				strEnd = WeightValue + strBegin;
				String MODString=code.substring(strBegin,strEnd);
				CheckDigitValue=UPCEANCheck(MODString);
				codetmp = codetmp + CheckDigitValue;
				i = i + 3;
			}

			else if ( (i < dataLength - 2) && (code.charAt(i-1) == '~') && ((int)code.charAt(i)> 47) && ((int)code.charAt(i) < 58) )
			{
				CurrentCharNum = ((((int)code.charAt(i))-48)*100) + ((((int)code.charAt(i+1))-48)*10)  + (((int)code.charAt(i+2))-48);
				if ((CurrentCharNum > -1) && (CurrentCharNum < 255))
				{
					codetmp = codetmp + (char)(CurrentCharNum);
					i = i + 3;
				}
				else
				{
					codetmp = codetmp + code.charAt(i-1);
				}
			}
			
			else
			{
				codetmp = codetmp + code.charAt(i-1);
			}
		}
		CurrentCharNum = 0;
		code=codetmp;
		codetmp="";
		//dataLength = code.length();
	}

	//add in FNC1 if needed
	if ( (int)code.charAt(0) < 202) code = ((char) 202) + code;	
	dataLength = code.length();


	//Set defaults to codeset A
	set=set128B;
	int sum=103;

	//Perform conversions to encode ASCII 0 to 31 in the barcode
	if (Code128Set!='0') 
	{
		codetmp="";
		codeText="";
		for(int i = 1; i <= dataLength; i++ )  
		{
			//Get ASCII value of each character
			CurrentChar = (int)code.charAt(i-1);
			if((CurrentChar < 32) && (CurrentChar >= 0)) 
			{
				if (Code128Set=='A') codetmp = codetmp + (char)(CurrentChar + 96);
				//For code set B, switch to code A, process function & switch back
				if (Code128Set=='B') 
				{
					if (((int)code.charAt(i))<32) 
					{
						codetmp = codetmp + (char)201 + (char)(CurrentChar + 96) + (char)((int)code.charAt(i) + 96) + (char)200;
						i++;
					}
					else codetmp = codetmp + (char)201 + (char)(CurrentChar + 96) + (char)200;
				}
				//For code set C, switch to code A, process function & switch back
				if (Code128Set=='C') 
				{
					if (((int)code.charAt(i))<32) 
					{
						codetmp = codetmp + (char)201 + (char)201 + (CurrentChar + 64) + ((int)code.charAt(i) + 64) + "99";
						i++;
					}
					else codetmp = codetmp + (char)201 + (char)201 + (CurrentChar + 64) + "99";
				}
				//For tabs and returns in text, show only spaces, hide all others from text
				if((CurrentChar == 13)||(CurrentChar == 9)) codeText = codeText + "  ";
			}
			else 
			{
				codeText = codeText + (char)CurrentChar;
				codetmp = codetmp + (char)CurrentChar;
			}
		}
	}


	if (Code128Set=='0') 
	{
		set=set128B;
		codeText="";
		codetmp="";
		char C128Start = (char)204;
		char CurrentEncoding = 'B';
		// Here we select character set A, B or C for the START character
		CurrentCharNum = (int)code.charAt(0);
		if (CurrentCharNum < 32) C128Start = (char)203;
		if (CurrentCharNum > 31 && CurrentCharNum < 127) C128Start = (char)204;
		if ((dataLength > 3) && (CurrentCharNum>47 && CurrentCharNum<58) &&((int)code.charAt(1)>47 &&(int)code.charAt(1)<58) && ((int)code.charAt(2)>47 && (int)code.charAt(2)<58) && ((int)code.charAt(3)>47 && (int)code.charAt(3)<58)) C128Start = (char)205;
		// 202 is the FNC1, also check for ASCII 121 to 215
		if ( (CurrentCharNum == 202) || (CurrentCharNum > 211) ) C128Start = (char)205;
		if (C128Start == (char)203) 
		{
			CurrentEncoding = 'A';
			sum=103;
			paintChar(g,"bwbwbw","211412", doPaint);
		}
		if (C128Start == (char)204) 
		{
			CurrentEncoding = 'B';
			sum=104;
			paintChar(g,"bwbwbw","211214", doPaint);
		}
		if (C128Start == (char)205) 
		{
			CurrentEncoding = 'C';
			sum=105;
			paintChar(g,"bwbwbw","211232", doPaint);
		}
		for(int i = 1; i <= dataLength; i++ )  
		{
			// Get ASCII value of each character
			CurrentCharNum = (int)code.charAt(i-1);

			// Check for FNC1
			if ((i < dataLength-1) && ((CurrentCharNum == 202) || (CurrentCharNum > 211)) ) 
			{
				//Here we encode ASCII 202, the FNC1 into the codetmp
				codetmp= codetmp + (char)202;
			}

			// BDA ADDED V5.04 FOR FNC2 SUPPORT
			// Check for FNC2
			else if (CurrentCharNum == 197)
			{
				//Here we encode ASCII 197, the FNC2 into the codetmp
				if (CurrentEncoding == 'C') 
				{
					codetmp= codetmp + (char)200;
					CurrentEncoding = 'B';
				}
				codetmp= codetmp + (char)197;
			}

				// Check for switching to character set C
			else if (((i <= dataLength - 3) && ((CurrentCharNum>47) && (CurrentCharNum<58) && ((int)code.charAt(i)>47) && ((int)code.charAt(i)<58) && ((int)code.charAt(i+1)>47) && ((int)code.charAt(i+1)<58) && ((int)code.charAt(i+2)>47) && ((int)code.charAt(i+2)<58)))
				|| ((i <= dataLength - 1) && (CurrentCharNum>47) && (CurrentCharNum<58) && ((int)code.charAt(i)>47) && ((int)code.charAt(i)<58) && (CurrentEncoding == 'C'))) 
			{
				


	//////////////////////////////BRJ/////5/17/06////////////////////////////////////////////////////////////////////
         //brj
             //check to see if we have an odd number of numbers to encode,
             //if so stay in current set for 1 number and then switch to save space
	 		if (CurrentEncoding !='C')
				{
							
      	      int j = i;
		  	 			int Factor = 3;  //

             			while (j <= dataLength  && isDigit((code.charAt(j-1))))
			
										{
											Factor = (4 - Factor);
											j++;
					 					}
	
            	if (Factor == 1)  
										{
											//if so stay in current set for 1 character to save space
											codetmp = codetmp + code.charAt(i-1); 
											  
											i++;
                     }

							Factor = 0;
		  }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

				// Switch to set C if not already in it
				if (CurrentEncoding != 'C') 
				{
					codetmp= codetmp + (char)199;
					CurrentEncoding = 'C';
				}

				// Get the value of each number pair (ex: 5 and 6 = 5*10+6 =56)
				CurrentCharNum = ((((int)code.charAt(i-1))-48)*10) + (((int)code.charAt(i))-48);
				if((CurrentCharNum < 95) && (CurrentCharNum  >= 0))
					codetmp= codetmp + (char)(CurrentCharNum + 32);
				else if(CurrentCharNum > 94)
					codetmp= codetmp + (char)(CurrentCharNum + 100);
				i = i + 1;
			}
				// Check for switching to character set A           
			else if ((i <= dataLength) && ((CurrentCharNum < 32) || ((CurrentEncoding == 'A') && (CurrentCharNum < 96)))) 
			{
				// Switch to set A if not already in it
				if (CurrentEncoding != 'A') 
				{
					codetmp= codetmp + (char)201;
					CurrentEncoding = 'A';
				}
				if (CurrentCharNum < 32) 
				{
					codetmp= codetmp + (char)(CurrentCharNum + 96);
				}
				else if (CurrentCharNum > 31) 
				{
					codetmp= codetmp + (char)CurrentCharNum;
				}
			}
				// Check for switching to character set B
			else if ((i <= dataLength) && (CurrentCharNum > 31 && CurrentCharNum < 127)) 
			{
				// Switch to set B if not already in it
				if (CurrentEncoding != 'B') 
				{
					codetmp= codetmp + (char)200;
					CurrentEncoding = 'B';
				}
				codetmp= codetmp + (char)CurrentCharNum;
			}
		}

		// FORMAT TEXT FOR AIs
		for(int i = 1; i <= dataLength; i++ )  
		{
			// Get ASCII value of each character
			CurrentCharNum = (int)code.charAt(i-1);
			// Check for FNC1
			CorrectFNC = 0;
			if ( (i < dataLength-1) && ( (CurrentCharNum == 202) || (CurrentCharNum > 211) ) ) 
			{
				// It appears that there is an AI
				// Get the value of each number pair (ex: 5 and 6 = 5*10+6 =56)
				CurrentCharNum = ((((int)code.charAt(i))-48)*10) + (((int)code.charAt(i+1))-48);


				// Is 2 digit AI by entering ASCII 212?
				if ((int)code.charAt(i-1) == 212)  
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + ") ";
					i+=2;
					CorrectFNC=1;
				}

					// Is 3 digit AI by entering ASCII 213?
				else if ( (i < dataLength-2) && ((int)code.charAt(i-1) == 213) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + ") ";
					i+=3;
					CorrectFNC=1;
				}

					// Is 4 digit AI by entering ASCII 214?
				else if ( (i < dataLength-3) && ((int)code.charAt(i-1) == 214) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + ") ";
					i+=4;
					CorrectFNC=1;
				}

					// Is 5 digit AI by entering ASCII 215?
				else if ( (i < dataLength-4) &&  ((int)code.charAt(i-1) == 215) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + code.charAt(i+4) + ") ";
					i+=5;
					CorrectFNC=1;
				}

					// Is 6 digit AI by entering ASCII 216?
				else if ( (i < dataLength-5) &&  ((int)code.charAt(i-1) == 216) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + code.charAt(i+4) + code.charAt(i+5) + ") ";
					i+=6;
					CorrectFNC=1;
				}

					// Is 7 digit AI by entering ASCII 217?
				else if ( (i < dataLength-6) &&  ((int)code.charAt(i-1) == 217) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + code.charAt(i+4) + + code.charAt(i+5) + code.charAt(i+6) + ") ";
					i+=7;
					CorrectFNC=1;
				}


					// Is 2 digit AI auto-detected?
				else if ( (CurrentCharNum <= 30 && CurrentCharNum  >= 0) || (CurrentCharNum <=99 && CurrentCharNum  >= 90) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + ") ";
					i+=2;
					CorrectFNC=1;
				}

					// Is 3 digit AI auto-detected?
				else if ( (i < dataLength-2) && ( (CurrentCharNum <= 49 && CurrentCharNum  >= 40) || (CurrentCharNum <=25 && CurrentCharNum  >= 23) ) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + ") ";
					i+=3;
					CorrectFNC=1;
				}

					// Is 4 digit AI auto-detected?
				else if ( (i < dataLength-3) && ( (CurrentCharNum <= 81 && CurrentCharNum  >= 80) || (CurrentCharNum <=34 && CurrentCharNum  >= 31) ) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + ") ";
					i+=4;
					CorrectFNC=1;
				}

					// Default to 4 digit AI?
				else if ( (i < dataLength-3) &&  (CorrectFNC==0) ) 
				{
					codeText = codeText + " (" + code.charAt(i) + code.charAt(i+1) + code.charAt(i+2) + code.charAt(i+3) + ") ";
					i+=4;
				}

			}
			else if ((int)code.charAt(i-1) < 32) codeText = codeText + " ";
			else if ( ((int)code.charAt(i-1) > 31) && ((int)code.charAt(i-1) < 128) ) codeText = codeText + code.charAt(i-1);		
		}
	}

	if (Code128Set=='B') 
	{
		set=set128B;
		sum=104;
	}
	if (Code128Set=='C') 
	{
		set=set128C;
		sum=105;
		// Check for an even number of digits, add 0 if not even
		if((codetmp.length() % 2) == 1) 
		{
			codetmp = "0"+codetmp;
			codeText=codetmp;		
		}
	}

	// paint start char
	if (Code128Set=='B') paintChar(g,"bwbwbw","211214", doPaint);
	if (Code128Set=='C') paintChar(g,"bwbwbw","211232", doPaint);
	if (Code128Set=='A') paintChar(g,"bwbwbw","211412", doPaint);

	// paint char in string
	int w=1; // weight for checksum
	for (int i=0;i<codetmp.length();i++) 
	{
		c=""+codetmp.charAt(i);						

		if (Code128Set=='C') 
		{
			// charater set C encodes two characters at a time
			// 1st character
			cChar=""+c;
			i++;
			// 2nd character added to the end of cChar
			if (i<codetmp.length()) cChar=cChar+codetmp.charAt(i);

			pos=findInArray(set128C,cChar);
			if (pos>=0)  
			{
				paintChar(g,"bwbwbw",set128[pos], doPaint);
				sum=sum+(pos*w);}

		}
		else 
		{
			// set A or B
			pos=findInArray(set,c);
			if (pos>=0)  
			{
				paintChar(g,"bwbwbw",set128[pos], doPaint);
				sum=sum+(pos*w);}
		}
		w++;
	}
		

	if (checkCharacter) 
	{
		check=(int) mod(sum,103);
		// check char
		paintChar(g,"bwbwbw",set128[check],doPaint);
	}

	// paint stop char
	paintChar(g,"bwbwbwb","2331112", doPaint);
	
}


	/** paints a EAN8 code
	 */	
protected void paintEAN8(Graphics g, boolean doPaint) {


// The EAN 8 symbol encodes two flag characters, five data characters, and a check character. The physical structure of EAN 8 is as follows:

// - Left hand guard bars, encoded 101
// - Two flag characters, encoded as left hand A characters
// - First two data characters, encoded as left hand A characters
// - Center guard pattern, encoded 01010
// - Last three data characters, encoded as right hand characters
// - Check character, encoded as a right hand character
// - Right hand guard bars, encoded 101



	  int pos=0;
	  String c;
	  String c2;
	  int sum=0;
	  String CodeTemp = ""; //tb 7/26/2006
	  
	      //Quick check to test for improper character. //brj 9/14/06
		    	for (int i=0;i<code.length();i++) 
			{
				if (((int)code.charAt(i)< 48) || ((int)code.charAt(i) > 57) )
				
				{
					codeText="Incorrect Data To Encode";	
					return;
				}
			}
	  
	  
	  if (code.length()<7) return;
	  
	  if(code.length()==9) {
		supplement=code.substring(7,9);
		UPCEANSupplement2=true;
		UPCEANSupplement5=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==10) {
		supplement=code.substring(8,10);
		UPCEANSupplement2=true;
		UPCEANSupplement5=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==12) {
		supplement=code.substring(7,12);
		UPCEANSupplement5=true;
		UPCEANSupplement2=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==13) {
		supplement=code.substring(8,13);
		UPCEANSupplement5=true;
		UPCEANSupplement2=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	 
	  //if ((code.length()==7) && (checkCharacter)) code=code+UPCEANCheck(code);
	  CodeTemp = code.substring(0,7);
	  CodeTemp=CodeTemp+UPCEANCheck(CodeTemp);
	  
	  // paint start char
	  paintGuardChar(g,"bwb","nnn",0, doPaint);
	  leftGuardBar=currentX;
	  
	  // paint char in string
	  for (int i=0;i<CodeTemp.length();i++) {
		  c=""+CodeTemp.charAt(i);

		  pos=-1;
		  if (i<=3) {

			pos=findChar(setEANLeftA,c);
			paintChar(g,"wbwb",setEANLeftA[pos][1], doPaint);
		  }
		  else {
			pos=findChar(setEANRight,c);
			paintChar(g,"bwbw",setEANRight[pos][1], doPaint);
		  }

		  // center guard Pattern
		  if (i==3)  {
			   centerGuardBarStart=currentX;
			   paintGuardChar(g,"wbwbw","nnnnn",0, doPaint);
			   centerGuardBarEnd=currentX;
		  }
	  }

	  rightGuardBar=currentX;

	  // paint stop char
	  paintGuardChar(g,"bwb","nnn",0, doPaint);
	  
	  endOfCode=currentX;
	  
	  codeText = CodeTemp;
	  // supplement
	  if (UPCEANSupplement2) paintSup2(g,supplement,doPaint);
	  else if (UPCEANSupplement5) paintSup5(g,supplement,doPaint);	  	  

	}

	/** paints a UPCE code
	 */	
protected void paintUPCE(Graphics g, boolean doPaint) {

// UPCE 
// UPCE 
// UPCE 
// UPCE 
// To convert a UPC A number to UPC E there must be at least four zeros in the data characters.
// The physical structure of the UPCE symbol is as follows:

// Left hand guard bars, encoded 101
// Six explicit data characters, encoded from the parity table.
// Right hand guard bars, encoded 010101
// To determine how to encode the data characters, first the UPC-A check character must be found.
// 	Once known, it is used to set the pattern of even and odd parity to be used to encode the six explicit
// 	characters from the UPC E Parity Table.

// Determining the explicit characters
//  1- If the manufacturers number ends in 000, 100, or 200, valid item numbers are 00000 to 00999.
//	  The six explicit characters are the first two characters of the manufacturer's number, the last three
//	  characters of the item number, and the third character of the manufacturer's number.
//  2- If the manufacturers number ends in 300, 400, 500, 600, 700, 800 or 900, valid item numbers are
//     00000 to 00099. The six explicit characters are the first three characters of the manufacturer's
//	    number, the last two characters of the item number, and the character ``3''.
//  3- If the manufacturers number ends in 10, 20, 30, 40, 50, 60, 70, 80, or 90, valid item numbers are 00000 to
//	     00009. The six explicit characters are the first four characters of the manufacturer's number, the
//	    last character of the item number, and the character ``4''.
//  4- If the manufacturers number does not end in zero, the valid numbers are 00005 to 00009. The six explicit
//      	characters are all five characters of the manufacturer's number, and the last character of the item
//      	number

	  int pos=0;
	  String c;
	  String c2;
	  int sum=0;
	  int checkchar=0;
	  String codetmp="";
	  String[][] setLeft;
	  String CodeTemp = ""; //tb 7/26/2006
	  
	  
	      //Quick check to test for improper character. //brj 9/14/06
		    	for (int i=0;i<code.length();i++) 
			{
				if (((int)code.charAt(i)< 48) || ((int)code.charAt(i) > 57) )
				
				{
					codeText="Incorrect Data To Encode";	
					return;
				}
			}
	  
	//check for add-ons and if they exist set them up in the barcode
	if(code.length()==13) {
		supplement=code.substring(11,13);
		UPCEANSupplement2=true;
		UPCEANSupplement5=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==14) {
		supplement=code.substring(12,14);
		UPCEANSupplement2=true;
		UPCEANSupplement5=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==16) {
		supplement=code.substring(11,16);
		UPCEANSupplement5=true;
		UPCEANSupplement2=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	if(code.length()==17) {
		supplement=code.substring(12,17);
		UPCEANSupplement5=true;
		UPCEANSupplement2=false; //tb 7/26/2006
		//code=code.substring(0,11);
	}
	//added by TB 6/06
	//upce 6, 7, and 8 digit support
	if(code.length()==6 || code.length()==7)
		code = UPCe7To11(code);
	if(code.length() ==8)
		code = UPCe7To11(code.substring(0,7));

	  // if (code.length()==11) code=code+UPCEANCheck(code);
	  // add in the check digit
	  if (code.length()<11) return;
	  //code=(code.substring(0,11))+UPCEANCheck(code);

	//add in the check digit
      //this code fixes a bug that would auto-increment the check digit in the applet
	CodeTemp=code.substring(0,11);
      CodeTemp=CodeTemp + UPCEANCheck(CodeTemp);

	  checkchar=(new Integer(""+CodeTemp.charAt(11)).intValue());
	
	  // case one
	  if ((CodeTemp.substring(3,6).compareTo("000")==0) || (CodeTemp.substring(3,6).compareTo("100")==0) || (CodeTemp.substring(3,6).compareTo("200")==0)) {
		  codetmp=CodeTemp.substring(1,3)+CodeTemp.substring(8,11)+CodeTemp.charAt(3);
	  }
	  // case two
	  if ((CodeTemp.substring(3,6).compareTo("300")==0) || (CodeTemp.substring(3,6).compareTo("400")==0) ||(CodeTemp.substring(3,6).compareTo("500")==0) ||(CodeTemp.substring(3,6).compareTo("600")==0) ||(CodeTemp.substring(3,6).compareTo("700")==0) ||(CodeTemp.substring(3,6).compareTo("800")==0) ||(CodeTemp.substring(3,6).compareTo("900")==0)) {
		  codetmp=CodeTemp.substring(1,4)+CodeTemp.substring(9,11)+"3";
	  }
	  // case three
	  if ((CodeTemp.substring(4,6).compareTo("10")==0) || (CodeTemp.substring(4,6).compareTo("20")==0) ||(CodeTemp.substring(4,6).compareTo("30")==0) ||(CodeTemp.substring(4,6).compareTo("40")==0) ||(CodeTemp.substring(4,6).compareTo("50")==0) ||(CodeTemp.substring(4,6).compareTo("60")==0) ||(CodeTemp.substring(4,6).compareTo("70")==0) ||(CodeTemp.substring(4,6).compareTo("80")==0) ||(CodeTemp.substring(4,6).compareTo("90")==0)) {
		  codetmp=CodeTemp.substring(1,5)+CodeTemp.substring(10,11)+"4";
	  }
	  // case four
	  if ((CodeTemp.substring(5,6).compareTo("0")!=0)) {
		  codetmp=CodeTemp.substring(1,6)+CodeTemp.substring(10,11);
	  }

        //codeText="0" + codetmp + checkchar;
        //codeText=codetmp;
        //codeText="3330530";

	  // paint start char
	  paintGuardChar(g,"bwb","nnn",0, doPaint);

	  leftGuardBar=currentX;

	  String System=UPCESystem0[checkchar];
	  codeText="0" + codetmp + checkchar;
	  if (UPCESytem=='1') 
	  {
	     System=UPCESystem1[checkchar];
		 codeText="1" + codetmp + checkchar;
	  }
	  if (UPCESytem=='2')
	  {
		  if (CodeTemp.substring(0,1).equals("0"))
		  {
			  System=UPCESystem0[checkchar];
			  codeText="0" + codetmp + checkchar;
		  }
		  if (CodeTemp.substring(0,1).equals("1"))
		  {
			  System=UPCESystem1[checkchar];
			  codeText="1" + codetmp + checkchar;
		  }
	  }
	 
	  // paint char in string
	  for (int i=0;i<codetmp.length();i++) {
		  c=""+codetmp.charAt(i);

		  pos=-1;

		  setLeft=setUPCEOdd;
		  if (System.charAt(i)=='E') setLeft=setUPCEEven;

		  pos=findChar(setLeft,c);
		  paintChar(g,"wbwb",setLeft[pos][1],doPaint);

	  }

	  rightGuardBar=currentX;

	  // paint stop char
	  paintGuardChar(g,"wbwbwb","nnnnnn",0, doPaint);
	  
	  endOfCode=currentX;
	  
	  // supplement
	  if (UPCEANSupplement2) paintSup2(g,supplement, doPaint);
	  else if (UPCEANSupplement5) paintSup5(g,supplement, doPaint);	  

	}




	/** paints a 2 digit supplement
	 */	
	protected void paintSup2(Graphics g,String chars, boolean doPaint) {
		
	  if (supplement.length()>0) chars=supplement;	
		
		
	  int i;
	  suplementTopMargin=(int) (barHeightPixels*(1-supHeight));	
	  //if (codeSup.length()==0) codeSup=chars;
	  codeSup=chars;
	  
	  
	  if (chars.length()!=2) return;	
		
	  currentX=(int) (currentX+(resolution*supSeparationCM)); // separation between code bar and supplement 
	  
	  startSuplement=currentX;
	  
	  
	  try {
	    i=Integer.valueOf(chars).intValue();
	  } catch (Exception e) {i=0;}
	  
	  String Parity="OO";
	  if (mod(i,4)==1) Parity="OE";
	  if (mod(i,4)==2) Parity="EO";
	  if (mod(i,4)==3) Parity="EE";
	  
	  // start
	  paintGuardChar(g,"bwb","112",suplementTopMargin, doPaint);
	
	  // first char
	  String[][] set=setUPCEOdd;
	  if (Parity.charAt(0)=='E') set=setUPCEEven;
	  int pos=findChar(set,""+chars.charAt(0));
	  paintGuardChar(g,"wbwb",set[pos][1],suplementTopMargin, doPaint);	  
	  
	  // delineator
	  paintGuardChar(g,"wb","11",suplementTopMargin, doPaint);
	  
	  // second char
	  set=setUPCEOdd;
	  if (Parity.charAt(1)=='E') set=setUPCEEven;
	  pos=findChar(set,""+chars.charAt(1));
	  paintGuardChar(g,"wbwb",set[pos][1],suplementTopMargin, doPaint);	  
	  
	  endSuplement=currentX;
	
	}
	
		/** paints a 5 digit supplement
	 */	
	protected void paintSup5(Graphics g,String chars, boolean doPaint) {
		
		if (supplement.length()>0) chars=supplement;
		
	  suplementTopMargin=(int) (barHeightPixels*(1-supHeight));	
	  //if (codeSup.length()==0) codeSup=chars;
	  codeSup=chars;
		
	  if (chars.length()!=5) return;	
	  
      // To determine the parity pattern for the five digit supplementals, use the following steps: 
      //
      // Designate the rightmost character odd 
      // Sum all of the characters in the odd positions and multiply the result by three. 
      // Sum all of the characters in the even positions and multiply the result by nine. 
      // Add the results from steps two and three. The number in the units position is the number of the parity pattern.  
	  
	 boolean odd=true;
	 int sumodd=0;
	 int sum=0;
	 int c;

	 for (int i=(chars.length()-1);i>=0;i--) {
		if (odd) sumodd=sumodd+(new Integer(""+chars.charAt(i)).intValue());
		else sum=sum+(new Integer(""+chars.charAt(i)).intValue());
		odd=(!odd);
	 }

	 sum=(sumodd*3)+(sum*9);

	 String sumstr=""+sum; // convert result to string
	 
	 // take last digit
     c=(new Integer(""+sumstr.charAt(sumstr.length()-1)).intValue()); 
	  
	 String Parity=fiveSuplement[c];
		
	 currentX=(int) (currentX+(resolution*supSeparationCM)); // separation between code bar and supplement 
	  
	 startSuplement=currentX;	
	 
	  // start
	  paintGuardChar(g,"bwb","112",suplementTopMargin, doPaint);
	  String[][] set=null;
	  int pos;
	  
	  for (int j=0;j<5;j++) {
	  
	    // first char
	    set=setUPCEOdd;
	    if (Parity.charAt(j)=='E') set=setUPCEEven;
	    pos=findChar(set,""+chars.charAt(j));
	    paintGuardChar(g,"wbwb",set[pos][1],suplementTopMargin, doPaint);	  
	  
	    // delineator
	    if (j<4) paintGuardChar(g,"wb","11",suplementTopMargin, doPaint);
	  
	  } 
	  
	  endSuplement=currentX;
	
	}	
	
	
	/** paints a matrix 2 of 5 code
	 */	
protected void paintMAT25(Graphics g, boolean doPaint) {


	  int pos=0;
	  String c;
	  String c2;
	  int sum=0;
	  String codetmp=code;

	  // paint start char
	  paintChar(g,"bwbwbw","wnnnnn", doPaint);

	  // paint char in string
	  for (int i=0;i<codetmp.length();i++) {
		  c=""+code.charAt(i);
		  pos=findChar(set25,c);
		  if (pos>=0)  paintChar(g,"bwbwbw",set25[pos][1]+"n", doPaint);
	  }


	  // paint stop char
	  paintChar(g,"bwbwbw","wnnnnn", doPaint);

	}

/** paints a code 39 code.
*/	
	protected void paintCODE39(Graphics g, boolean doPaint) {

    // Code 39 is an alphanumeric bar code that can encode decimal numbers,
    //the upper case alphabet, and the following special symbols:

    //               _    .    *    $    /    %    +

    //Code 39 characters are constructed using nine elements, five bars and
    //four spaces. Of these nine elements, two of the bars and one of the spaces are
    //wider than the rest. Wide elements represent binary ones (1), and narrow elements
    //represent binary zeros (0). The character set table shows each of the available
    //characters with their corresponding check character values.

    //To enable a decoder to distinguish between the wide and narrow elements a minimum
    //wide to narrow ratio is needed. Depending upon which resolution has been used for
    //the printing of the bar code, the width of the wide element should be at least two
	//times greater than the narrow element. A ratio of three to one is better. All
    //elements of the same type should be printed the same size. (The width of a narrow
    //bar should be the same as a narrow space.)


    //The Code 39 check character is a modulus 43 sum of all of the message character
    //values and is printed as the last character in the message. The check character
    //values are given in the above table. The check chararacter is computed by adding
    //up all of the values, dividing by 43, and using the remainder as the value of the
    //check character.

	  int pos=0;
	  String c;
	  int sum=0;

          // move all lower case letters to uppercase
          code = code.toUpperCase();

	  // paint *
	  paintChar(g,"bwbwbwbwb",set39[findChar(set39,"*")][1], doPaint);

	  // interchar separator
	  //currentX=currentX+((int) (I*X*resolution));
	    currentX=currentX+narrowBarPixels;

	  // paint char in string
	  for (int i=0;i<code.length();i++) {
		  c=""+code.charAt(i);
		  pos=findChar(set39,c);
		  if (pos>-1) {
			  sum=sum+pos;
			  paintChar(g,"bwbwbwbwb",set39[pos][1],doPaint);
	          // interchar separator
	  	  //currentX=currentX+((int) (I*X*resolution));
	    	  currentX=currentX+narrowBarPixels;		  }
	  }

	  // check character
	  if (checkCharacter) {
		   pos=(int) mod(sum,43);
		   paintChar(g,"bwbwbwbwb",set39[pos][1],doPaint);
	           // interchar separator
		   //currentX=currentX+((int) (I*X*resolution));
	    	   currentX=currentX+narrowBarPixels;

		   //   codeText=code+""+set39[pos][0];
	           if (checkCharacterInText) codeText=code+""+set39[pos][0];
	           else codeText=code;
	  }

	  // paint *
	  paintChar(g,"bwbwbwbwb",set39[findChar(set39,"*")][1],doPaint);

	}

/** paints code 11 code	 */	
	protected void paintCODE11(Graphics g, boolean doPaint) {

	  // Code 11 uses two check digits, C and K. The first check digit C is the modulo
	  // 11 sum of the weighted products with the weights changing from 1 to 10.
      // The second check digit K is also the modulo 11 sum of the weighted products
      // with the weights changing from 1 to 9. The character - is assigned the value of 10.
      // As a rule of thumb, when the message length is 10 digits or less, usually only the C // check digit is used.


	  int pos=0;
	  String c;
	  int ch1;
	  int ch2;
	  int sum=0;

	  // paint start/Stop
	  paintChar(g,"bwbwbw","nnwwnn",doPaint);

	  // calculate checksum C
	  int w=1; //weight
	  sum=0;
	  for (int i=(code.length()-1);i>=0;i--) {
		  sum=sum+(findChar(set11,""+code.charAt(i))*w);
		  w++;
		  if (w==11) w=1;
	  }

	  ch1=(int) mod(sum,11);

	  // calculate checksum K
	  w=2; //weight
	  sum=ch1;
	  for (int i=(code.length()-1);i>=0;i--) {
		  sum=sum+(findChar(set11,""+code.charAt(i))*w);
		  w++;
		  if (w==10) w=1;
	  }

	  ch2=(int) mod(sum,11);


	  // paint char in string
	  for (int i=0;i<code.length();i++) {
		  c=""+code.charAt(i);
		  pos=findChar(set11,c);
		  if (pos>-1) {
			  paintChar(g,"bwbwbw",set11[pos][1]+"n",doPaint);
	  }
	  }

	  if (checkCharacter) {
	  // check character C
	  paintChar(g,"bwbwbw",set11[ch1][1]+"n",doPaint);

	  if (checkCharacterInText) codeText=code+set11[ch1][0];
	  else codeText=code;

	  // check character K, if requested or lentgh > 10
	  if (code.length()>10) {
		   paintChar(g,"bwbwbw",set11[ch2][1]+"n",doPaint);

	           if (checkCharacterInText) codeText=codeText+set11[ch2][0];
	           else codeText=code;
	  }
	 }

	  // paint start/Stop
	  paintChar(g,"bwbwbw","nnwwnn",doPaint);


	}
	

	/** paints a CODABAR code
	 */	
	protected void paintCODABAR(Graphics g, boolean doPaint) {

	 // Codabar is a discrete, numeric code with special characters and four different
	 // start/stop characters. Each character is encoded as seven elements, with two or
	 // three of the elements wide. While the Codabar check character is undefined, AIM
	 // has a recommended check character. The following table shows the Codabar character encodation.
	 // The numbers in the table correspond to the width of the element. A 0 means the
	 // element is narrow, and a 1 means the element is wide.
	 //
	 //While Codabar has no check character, AIM has a recommended check character.
	 //	Each Codabar character has a value assigned to it. (See above table).
	 //	The check character is computed as follows.
     //
     // 1. The sum of all character values is taken, including the Start and the Stop
	 //	characters.
     // 2. The data character whose value that when added to the total from step one
	 //	equals a multiple of 16 is the check character.


	  int pos=0;
	  String c;
	  int ch1;
	  int sum=0;

	  // paint start/Stop, A
	  paintChar(g,"bwbwbwbw",setCODABAR[findChar(setCODABAR,""+CODABARStartChar)][1]+"n",doPaint);

	  // calculate checksum C
	  sum=findChar(setCODABAR,""+CODABARStartChar)+findChar(setCODABAR,""+CODABARStopChar);
	  for (int i=(code.length()-1);i>=0;i--) {
		  sum=sum+(findChar(setCODABAR,""+code.charAt(i)));
	  }

	  ch1=(int) mod(sum,16);
	  if (ch1!=0) ch1=16-ch1;

	  // paint char in string
	  for (int i=0;i<code.length();i++) {
		  c=""+code.charAt(i);
		  pos=findChar(setCODABAR,c);
		  if (pos>-1) {
			  paintChar(g,"bwbwbwbw",setCODABAR[pos][1]+"n",doPaint);
	  }
	  }

	  // check character, if requested
	  if (checkCharacter)  {
		   // codeText=code+setCODABAR[ch1][0];
	           if (checkCharacterInText) codeText=code+setCODABAR[ch1][0];
	           else codeText=code;
		   paintChar(g,"bwbwbwbw",setCODABAR[ch1][1]+"n",doPaint);
	  }

	  // paint start/Stop, B
	  paintChar(g,"bwbwbwb",setCODABAR[findChar(setCODABAR,""+CODABARStopChar)][1],doPaint);


	}

/** paints a MSI code
 */
protected void paintMSI(Graphics g, boolean doPaint) {

	 // MSI Code is a numeric, continuous code. Each character consists of four bits,
	 //	using a straight binary encodation. Each bit is encoded in the following way:
	 //	if the bit is a 1, the pattern to be used is a wide bar followed by a narrow
	 //	space. If the bit is to be a 0, the pattern is a narrow bar followed by a
	 //	wide space. The following table is the bit patterns:
	 //
	 // The start character is a single ``1'' bit. (wide bar/narrow space).
	 //	The stop character is a narrow bar/wide space/narrow bar.
     // The check digit is calculated by using the following steps.

     // 1. Starting from the units position, create a new number with all of the odd position digits in their original sequence.
     // 2. Multiply this new number by 2.
     // 3. Add all of the digits of the product from step two.
     // 4. Add all of the digits not used in step one to the result in step three.
     // 5. Determine the smallest number which when added to the result in step four will result in a multiple of 10. This is the check character.



	  int pos=0;
	  String c;
	  int ch1;
	  int sum=0;
	  
	  		    //Quick check to test for improper character. //brj 9/14/06
		    	for (int i=0;i<code.length();i++) 
			{
				if (((int)code.charAt(i)< 48) || ((int)code.charAt(i) > 57) )
				
				{
					codeText="Incorrect Data To Encode";	
					return;
				}
			}	

	  // paint start
	  paintChar(g,"bw","wn",doPaint);

	  // calculate checksum C
	  sum=0;
	  String oddNumber="";
	  boolean odd=true;
	  for (int i=(code.length()-1);i>=0;i--) {
		  if (!odd) sum=sum+(findChar(setMSI,""+code.charAt(i)));
		  if (odd) oddNumber=(findChar(setMSI,""+code.charAt(i)))+oddNumber;
		  odd=(!odd);
	  }

	 oddNumber=""+(new Long(oddNumber).longValue())*2;
	 for (int i=(oddNumber.length()-1);i>=0;i--) {
		 sum=sum+(findChar(setMSI,""+oddNumber.charAt(i)));
	 }

	  ch1=(int) mod(sum,10);
	  if (ch1!=0) ch1=10-ch1;

	  // paint char in string
	  for (int i=0;i<code.length();i++) {
		  c=""+code.charAt(i);
		  pos=findChar(setMSI,c);
		  if (pos>-1) {
			  paintChar(g,"bwbwbwbw",setMSI[pos][1],doPaint);
	  }
	  }

	  // check character K, if requested or lentgh > 10
	  if (checkCharacter) {
		   paintChar(g,"bwbwbwb",setMSI[ch1][1],doPaint);

		   // codeText=code+setMSI[ch1][0];
	           if (checkCharacterInText) codeText=code+setMSI[ch1][0];
	           else codeText=code;
	  }

	  // paint stop
	  paintChar(g,"wbwb","nnwn",doPaint);


	}

	protected static double mod(double a, double b) {
		double f;

		f= (double) (a/b);

		double i = Math.round(f);
		if (i>f) i=i-1;

		return (a - (b*i));



	}




	/** paints a code 39 extended code
 */
protected void paintCODE39Ext(Graphics g, boolean doPaint) {

	  int pos=0;
	  String c;
	  String encoded;
	  byte b;
	  int sum=0;

	  
	   // move all lower case letters to uppercase  //brj 5/5/2006
	       //     code = code.toUpperCase();

	  // paint *
	  paintChar(g,"bwbwbwbwb",set39[findChar(set39,"*")][1],doPaint);
	  
	  

	  // interchar separator
	  currentX=currentX+narrowBarPixels;

	  // paint char in string
	  for (int i=0;i<code.length();i++) {
		  b=(byte) code.charAt(i);
		  if (b<=128) {
                    encoded=set39Ext[b];
                    for (int j=0;j<encoded.length();j++) {
                      c=""+encoded.charAt(j);
                      pos=findChar(set39,c);
                      if (pos>-1) {
			   sum=sum+pos;
			   paintChar(g,"bwbwbwbwb",set39[pos][1],doPaint);
	                   // interchar separator
			   currentX=currentX+narrowBarPixels;
		     }
		  }
	    }
	  }

             //Remove ASCII 0 to 31 from the text to display
             codeText="";
             int CurrentChar=0;
             for(int i = 1; i <= code.length(); i++ )  {
               //Get ASCII value of each character
               CurrentChar = (int)code.charAt(i-1);
               if((CurrentChar < 32) && (CurrentChar >= 0)) {
                  //For tabs and returns in text, show only spaces, hide all others from text
                  if((CurrentChar == 13)||(CurrentChar == 9)) codeText = codeText + "  ";
                  }
               else {
                  codeText = codeText + (char)CurrentChar;
                  }
               }

	  // check character
	  if (checkCharacter) {
		   pos=(int) mod(sum,43);
		   paintChar(g,"bwbwbwbwb",set39[pos][1],doPaint);
	           // interchar separator
		   currentX=currentX+narrowBarPixels;
		   // codeText=code+""+set39[pos][0];
	           if (checkCharacterInText) codeText=codeText+""+set39[pos][0];
	  }

	  // paint *
	  paintChar(g,"bwbwbwbwb",set39[findChar(set39,"*")][1],doPaint);

	}

////////////////////////////////////////////////////////////////////////CODE 93//////////////////////////////////////////////////////////

/** paints a code 93 barcode
 */

protected void paintBAR93(Graphics g, boolean doPaint) {
      
	  int pos=0;
	  String c;
	  String encoded;
	  byte b;
	  int sum=0;
	  int ch2=0;
	  int ch1=0;


	  // move all lower case letters to uppercase  //brj 5/3/2006
    code = code.toUpperCase();

	  // paint start
	  paintChar(g,"bwbwbw","111141",doPaint);

	  // paint char in string
	  for (int i=0;i<code.length();i++) {
		    c= ""+code.charAt(i);   
		    pos=findChar(set93,c);
                                  
			if (pos>-1) {
			  sum=sum+pos;
			  paintChar(g,"bwbwbw",set93[pos][1],doPaint);
			}
	        // interchar separator
			//currentX=currentX+((int) (I*X*resolution));	 
		}

      // calculate checksum C
	  int w=1; //weight
	  sum=0;
	  for (int i=(code.length()-1);i>=0;i--) {
		  sum=sum+(findChar(set93,""+code.charAt(i))*w);
		  w++;
		  if (w==21) w=1;
	  }

	  ch1=(int) mod(sum,47);

	  // calculate checksum K
	  w=2; //weight
	  sum=ch1;
	  for (int i=(code.length()-1);i>=0;i--) {
		  sum=sum+(findChar(set93,""+code.charAt(i))*w);
		  w++;
		  if (w==16) w=1;
	  }

	  ch2=(int) mod(sum,47);



	  // check character
	  if (checkCharacter) {	   
		   paintChar(g,"bwbwbw",set93[ch1][1],doPaint);
		   paintChar(g,"bwbwbw",set93[ch2][1],doPaint);

		   // codeText=code+set93[ch1][0].charAt(0)+set93[ch2][0].charAt(0); 
	     //if (checkCharacterInText) codeText=code+set93[ch1][0].charAt(0)+set93[ch2][0];   //.charAt(0); 
			 if (checkCharacterInText) codeText=code+set93[ch1][0]+set93[ch2][0];   //.charAt(0);                            
	     else codeText=code; 
	  }
                
       // paint stop
	  	paintChar(g,"bwbwbwb","1111411",doPaint);
	  
				//will convert all "_1","_2","_3","_4" to their proper Code 93 character set characters. brj
	  String tempCodeText = "";
	  for (int i=0;i<codeText.length();i++) {
	  	if (codeText.charAt(i)=='_' && codeText.charAt(i+1)=='1') 
	  	{
	  		tempCodeText+="!";
	  		i++;
	  	}
	  	else if (codeText.charAt(i)=='_' && codeText.charAt(i+1)=='2') 
	  	{
	  		tempCodeText+="#";
	  		i++;
	  	} else if (codeText.charAt(i)=='_' && codeText.charAt(i+1)=='3') 
	  	{
	  		tempCodeText+="&";
	  		i++;
	  	} else if (codeText.charAt(i)=='_' && codeText.charAt(i+1)=='4') 
	  	{
	  		tempCodeText+="@";
	  		i++;
	  	}else tempCodeText+= codeText.charAt(i);
	  }
	  codeText = tempCodeText;
	}
    	 
////////////////////////////////////////////////////////////CODE 93 END/////////////////////////////////////////////////////////////////



/** paints a code 93 extended barcode
 */
	protected void paintBAR93Ext(Graphics g, boolean doPaint) {

	  int pos=0;
	  String c;
	  String encoded;
	  byte b;
	  int sum=0;
	  int ch2=0;
	  int ch1=0;


	 // move all lower case letters to uppercase  //brj 5/5/2006
	            //code = code.toUpperCase();

	  // paint start
	  paintChar(g,"bwbwbw","111141",doPaint);
	  


	  // paint char in string
	  for (int i=0;i<code.length();i++) {
		  b=(byte) code.charAt(i);
		  if (b<=128) {
			encoded=set93Ext[b];

			if (encoded.length()==3) {
				// control char, _1,_2,_3, or _4
		       c=""+encoded.charAt(0)+encoded.charAt(1);
		       pos=findChar(set93,c);
			   paintChar(g,"bwbwbw",set93[pos][1],doPaint);
			   
	           // interchar separator
			   //currentX=currentX+((int) (I*X*resolution));

			   c=""+encoded.charAt(2);
			}
			else c=""+encoded.charAt(0);

		    pos=findChar(set93,c);
			sum=sum+pos;
			paintChar(g,"bwbwbw",set93[pos][1], doPaint);
	        // interchar separator
			//currentX=currentX+((int) (I*X*resolution));
	    }
	  }
	  	  
     // calculate checksum C
	  int w=1; //weight
	  sum=0;
	  for (int i=(code.length()-1);i>=0;i--) { 
	      b=(byte) code.charAt(i);
		  if (b<=128) {
			encoded=set93Ext[b];

			if (encoded.length()==3) {
			   	// control char, _1,_2,_3, or _4
		       c=""+encoded.charAt(0)+encoded.charAt(1);
		       pos=findChar(set93,c);
			   sum=sum+(pos*(w+1));

			   c=""+encoded.charAt(2);
			   pos=findChar(set93,c);
			   sum=sum+(pos*w);
		       w++;	
			   if (w==21) w=1;
			   w++;
			   if (w==21) w=1;		   
			}
			
			else {
				 c=""+encoded.charAt(0);
		         pos=findChar(set93,c);
			     sum=sum+(pos*w);
		         w++;	
				 if (w==21) w=1;
			}	
		  }
	  }

	  ch1=(int) mod(sum,47);

	  // calculate checksum K
	  w=2; //weight
	  sum=ch1;
	  for (int i=(code.length()-1);i>=0;i--) { 
	      b=(byte) code.charAt(i);
		  if (b<=128) {
				encoded=set93Ext[b];

				if (encoded.length()==3) {
			  	// control char, _1,_2,_3, or _4
		    	c=""+encoded.charAt(0)+encoded.charAt(1);
		    	pos=findChar(set93,c);
			  	sum=sum+(pos*(w+1));

			 		c=""+encoded.charAt(2);
			   	pos=findChar(set93,c);
			   	sum=sum+(pos*w);
		      w++;	
			   	if (w==16) w=1;			   		
			   	w++;	
			   	if (w==16) w=1;			   
				}
				else {
				 	c=""+encoded.charAt(0);
		      pos=findChar(set93,c);
			    sum=sum+(pos*w);
		      w++;	
					if (w==16) w=1;
				}	
		  }
	  }

	  ch2=(int) mod(sum,47);  

	  // check character
	  if (checkCharacter) {
		   paintChar(g,"bwbwbw",set93[ch1][1],doPaint);
		   paintChar(g,"bwbwbw",set93[ch2][1],doPaint);

		   // codeText=code+set93[ch1][0].charAt(0)+set93[ch2][0].charAt(0);
	     /* if (checkCharacterInText) codeText=code+set93[ch1][0].charAt(0)+set93[ch2][0].charAt(0);
	        else codeText=code;
	     */
			if (checkCharacterInText) codeText=code+set93[ch1][0]+set93[ch2][0];
	  	else codeText=code;
	  }

       // paint stop
	  paintChar(g,"bwbwbwb","1111411",doPaint);
	  String codeTextTemp = codeText.substring(codeText.length()-5);
	  //will convert all "_1","_2","_3","_4" to their proper Code 93 character set characters. brj
		for (int i=codeText.length(); i<codeText.length(); i++) {
			if (codeText.charAt(i)=='_' && codeText.charAt(i+1)=='1'){
				codeTextTemp+="!";
				i++;
			} else if (codeText.charAt(i)=='_' && codeText.charAt(i+1)=='2'){
				codeTextTemp+="#";
				i++;
			}else if (codeText.charAt(i)=='_' && codeText.charAt(i+1)=='3'){
				codeTextTemp+="&";
				i++;
			}else if (codeText.charAt(i)=='_' && codeText.charAt(i+1)=='4'){
				codeTextTemp+="@";
				i++;
			} else codeTextTemp +=codeText.charAt(i);
		}
		codeText = codeTextTemp;
			
	  //codeText = codeText.replaceAll("_1","!");
    //codeText = codeText.replaceAll("_2","#");
    //codeText = codeText.replaceAll("_3","&");
    //codeText = codeText.replaceAll("_4","@");
	}
	
/** paints a character
 */
	protected void paintChar(Graphics g,String patternColor,String patternBars, boolean doPaint)  {
		
		paintChar2(g,patternColor,patternBars,0,doPaint);

	}
/** paints a character
 */	
	protected void paintChar2(Graphics g,String patternColor,String patternBars,int bartopmargin,boolean doPaint)  {
		char cColor;
		char cBar;

		for (int i=0;i<patternColor.length();i++) {
			cColor=patternColor.charAt(i);
			cBar=patternBars.charAt(i);
			if (cBar=='n') addBar(g,narrowBarPixels,(cColor=='b'),bartopmargin,doPaint);
			if (cBar=='w') addBar(g,widthBarPixels,(cColor=='b'),bartopmargin,doPaint);
			if (cBar=='1') addBar(g,narrowBarPixels,(cColor=='b'),bartopmargin,doPaint);
			if (cBar=='2') addBar(g,(narrowBarPixels*2),(cColor=='b'),bartopmargin,doPaint);
			if (cBar=='3') addBar(g,(narrowBarPixels*3),(cColor=='b'),bartopmargin,doPaint);
			if (cBar=='4') addBar(g,(narrowBarPixels*4),(cColor=='b'),bartopmargin,doPaint);

		}

	}	
/** paints a guard character
 */	
protected void paintGuardChar(Graphics g,String patternColor,String patternBars,int bartopMargin,boolean doPaint)  {

		// Guard bars are longer that the other
		if ((textFont!=null) && (guardBars)) {
			//tb 8/11/2006, font size based on resolution
	    g.setFont(new Font(textFont.getName(),textFont.getStyle(),(int)(textFont.getSize()*resolution*.027)));
	    //tb 8/11/2006, guaurd bar should be half the height of the text
			extraHeight=g.getFontMetrics().getHeight()/2;
	    }
		paintChar2(g,patternColor,patternBars, bartopMargin, doPaint);
		extraHeight=0;

	}


	private String UPCe7To11(String DataToExpand)
	{
		//2006.6 TB added this function
		//This function expands 7 digits to the UPC-A 11 needed
		//If 6 digits entered, assume the number system is 0
		if(DataToExpand.length()==6)
			DataToExpand = "0" + DataToExpand;

		if(DataToExpand.length()!=7)
			DataToExpand="0000000";

		//Digits of DataToExpand
		int D7case = Integer.valueOf(DataToExpand.substring(6,7)).intValue();//switch case needs int value
		String D1 = DataToExpand.substring(0,1);
		String D2 = DataToExpand.substring(1,2);
		String D3 = DataToExpand.substring(2,3);
		String D4 = DataToExpand.substring(3,4);
		String D5 = DataToExpand.substring(4,5);
		String D6 = DataToExpand.substring(5,6);
		String D7 = DataToExpand.substring(6,7);
		
		switch (D7case)
		{
			case 0:
				DataToExpand = D1 + D2 + D3 + "00000" + D4 + D5 + D6;
				break;
			case 1:
				DataToExpand = D1 + D2 + D3 + D7 + "0000" + D4 + D5 + D6;
				break;
			case 2:
				DataToExpand = D1 + D2 + D3 + D7 + "0000" + D4 + D5 + D6;
				break;
			case 3:
				DataToExpand = D1 + D2 + D3 + D4 + "00000" + D5 + D6;
				break;
			case 4:
				DataToExpand = D1 + D2 + D3 + D4 + D5 + "00000" + D6;
				break;
			case 5:
				DataToExpand = D1 + D2 + D3 + D4 + D5 + D6 + "0000" + D7;
				break;
			case 6:
				DataToExpand = D1 + D2 + D3 + D4 + D5 + D6 + "0000" + D7;
				break;
			case 7:
				DataToExpand = D1 + D2 + D3 + D4 + D5 + D6 + "0000" + D7;
				break;
			case 8:
				DataToExpand = D1 + D2 + D3 + D4 + D5 + D6 + "0000" + D7;
				break;
			case 9:
				DataToExpand = D1 + D2 + D3 + D4 + D5 + D6 + "0000" + D7;
				break;
		}
		return DataToExpand;
	}
	/** calculates L and converts size in CM to pixels
 */	
	protected void calculateSizes() {
		int C=(code.length()); // lentgh of code in chars   
			
		
		
		byte b;

		narrowBarCM=X;
		widthBarCM=X*N;
/* we have a much more accurate way of doing this now
		 // length
		if (barType==INTERLEAVED25) {
			// leading 0 needed??
			if ((mod(C,2)==0) && (checkCharacter)) C++;
			if ((mod(C,2)==1) && (!checkCharacter)) C++;

			if (checkCharacter) C++;

			L=((C/2)*(3+(2*N))*X)+(7*X);
		}


		if (barType==UPCA) {
			//if (checkCharacter) C++;
			L=(C*7*X)+((3+3+5)*X);
		}

		if (barType==EAN13) {
			L=(C*7*X)+((3+3+5)*X);
		}

		if (barType==EAN8) {
			L=(C*7*X)+((3+3+5)*X);
		}

		if (barType==CODE128) {
			if (checkCharacter) C++;

            if (Code128Set=='C') L = ((11*C) + 35)*X ;
			else L = ((5.5*C) + 35)*X;
		}

		if (barType==UPCE) {
			// modified (was 6*7*X)+((3+6)*X) (added!!)
			L=(8*7*X)+((3+3+5)*X);
		}


		if (barType==IND25) {
			if (checkCharacter) C++;

			L=(C*(3+(2*N))*X)+(7*X);
		}

		if (barType==MAT25) {
			if (checkCharacter) C++;

			L=(C*(3+(2*N))*X)+(7*X);
		}


		if (barType==MSI) {
			if (checkCharacter) C++;

			L=(C*(4+(4*N))*X)+ ((1+N)*X)+((2+N)*X);
		}

		if (barType==CODABAR) {


			if (checkCharacter) C++;

			// almost all char have 3 narrow, 2 wide. (3+(2*N))
			L=((C+2)*(4+(3*N))*X);
		}

		if (barType==CODE11) {

			if ((checkCharacter) || (code.length()>10)) C++;

			// almost all char have 3 narrow, 2 wide. (3+(2*N))
			L=((C+2+1)*(3+(2*N))*X);

		}


		if (barType==POSTNET) {
			 if (checkCharacter) C++;

			 L=(X*10);			
		}
		
		if (barType==CODE39) {
			 if (checkCharacter) C++;

			 L=((C+2)*((3*N)+6)*X) +((C+1)*I*X);

		}

		if (barType==CODE39EXT) {
			String encoded;

			C=0;
			if (checkCharacter) C++;
			 // count chars needed to encode
			for (int j=0;j<code.length();j++) {
				b=(byte) code.charAt(j);
				if (b<=128) {
					encoded=set39Ext[b];
					C=C+encoded.length();
				}

			}

			 L=((C+2)*((3*N)+6)*X) +((C+1)*I*X);

		}

		if ((barType==CODE93) ||(barType==CODE93EXT)) {
			String encoded;

			C=0;
			if (checkCharacter) C++;
			 // count chars needed to encode
			for (int j=0;j<code.length();j++) {
				b=(byte) code.charAt(j);
				if (b<=128) {
					encoded=set39Ext[b];
					if (encoded.length()==1) C++;
					else C=C+2;
				}

			}

			 L=((C+2)*(9*X)) +((C+1)*I*X);

		}
*/
		// height
		if ( barHeightCM==0)  {
			barHeightCM=L*H;
			// minimum of 0.252 inches
			if (barHeightCM<(0.25*2.5)) barHeightCM=(0.25*2.5);
		}

		// ** calculate dimensions in pixels **
		if (barHeightCM!=0) barHeightPixels=(int) (barHeightCM*resolution);
		if (narrowBarCM!=0) narrowBarPixels=(int) (narrowBarCM*resolution);
		if (widthBarCM!=0) widthBarPixels= (int) (narrowBarPixels*N);
		if (narrowBarPixels<=0) narrowBarPixels=1;
		if (widthBarPixels<=1) widthBarPixels=2;
	}


// This method was changed in version 2.2
/** paints the barcode and rotates it if necessary
 */
	public void paint(Graphics g2) {
 		Graphics g=g2;
    Image im=null;
    textMarginPixels = (int) (textMarginCM * resolution);

   
    g.setFont(new Font("Arial",Font.PLAIN,11));
		int DemoPixelH = g.getFontMetrics().getHeight();
		//tb 8/11/2006, font size based on resolution
		g.setFont(new Font(textFont.getName(),textFont.getStyle(),(int)(textFont.getSize()*resolution*.027)));
		int TextH = g.getFontMetrics().getHeight();
	  	
    if (rotate!=0) {
  	// we will need to rotate the image, create a temporary image
    String v=java.lang.System.getProperty("java.version");
		   if ((v.indexOf("1.0")==0) ||(v.indexOf("1.1")==0))  {
		   	// in java 1.1 the barcode must be visible
		    im=this.createImage(this.getSize().width,this.getSize().height);
        g=im.getGraphics();
		   	}
		 	else  {                     
      	// in java 1.2 or newer we can create a buffered image
        IDAImageCreator imc=new IDAImageCreator();
        im=imc.getImage(this.getSize().width,this.getSize().height);
        g=imc.getGraphics();
        }
     	}

			// background color
			g2.setColor(backColor);
			g2.fillRect(0,0,this.getSize().width,this.getSize().height);
      paintBasis(g);		
			int maxw = 0; // area to rotate
			int maxh = 0;// left and top margin are also right and bottom
     	if (rotate!=0) {
    		// rotate and draw image in graphics
    		//maxw=currentX+leftMarginPixels+ ((int) ((fBearerBarVert * narrowBarPixels))*2);
    		maxw = pWidth;
    		if (!showText)
					textMarginPixels = 0;
					
    		//added by tb, include the textMarginPixels in the area to rotate
    		if (d==1){
					//maxh=currentY+topMarginPixels+textMarginPixels;
					maxh=currentY+topMarginPixels+textMarginPixels+((int) ((fBearerBarHori * narrowBarPixels))); //bens\
					
					if (barType == EAN8 || barType == EAN13 || barType == UPCE || barType == UPCA)
					   {
					   	//to ensure that offset for bearer bar is not cut off canvas in bearerbar function
					   	maxh+=(int)((fBearerBarVert * narrowBarPixels));
					  }
					  
					  
					  if (showText == false) {
						  maxh+=(int)((fBearerBarVert * narrowBarPixels));
						}
					
				}
				else{  //if demo include the height of powered by as well
					maxh=currentY+topMarginPixels+textMarginPixels+DemoPixelH+((int) ((fBearerBarHori * narrowBarPixels)));;
					
				
					
					
					if (barType == EAN8 || barType == EAN13 || barType == UPCE || barType == UPCA)
					   {
					   	//to ensure that offset for bearer bar is not cut off canvas in bearerbar function
					   	maxh+=(int)((fBearerBarVert * narrowBarPixels));
					  }
					  
					  if (showText == false) {
						  maxh+=(int)((fBearerBarVert * narrowBarPixels));
						}
					
				}
					
    		Image imRotated=rotate(im,rotate,maxw,maxh);
				if (imRotated==null) g2.drawImage(im,0,0,null); // rotation failed
    		else g2.drawImage(imRotated,0,0,null);
   		}
   		
	}
	   
/** paints the barcode
 */	
 	protected void paintBasis(Graphics g) {
 	 if (barType==ONECODE) {
		  if (resolution<40 && X<.06) setXDimensionCM(.06);
			if (X <.0381) setXDimensionCM(.0381);
			if (X >.0635) setXDimensionCM(.0635);
		}
		codeText="";
            
  	//6/06/06 brj
  	g.setFont(new Font("Arial",Font.PLAIN,11)); 
  	//demoPixelW is the width of the string
		demoPixelW = g.getFontMetrics().stringWidth("Powered by IDAutomation.com"); 
		
		calculateSizes();
  	int origLeftMarginPixels = 0;

		topMarginPixels= (int) (topMarginCM*resolution);             
		leftMarginPixels= (int) (leftMarginCM*resolution);
	
		origLeftMarginPixels = leftMarginPixels;
		
		//tb 7/06
		XdimPixels = (int) (X * resolution);
		WhiteBarIncreasePixels = (int) (WhiteBarIncrease * resolution);
		//tb 8/11/2006, font size based on resolution
 		g.setFont(new Font(textFont.getName(),textFont.getStyle(),(int)(textFont.getSize()*resolution*.027)));
		int TextH=g.getFontMetrics().getHeight();
		// background color
		g.setColor(backColor);
		int w =this.getSize().width;
		int h =this.getSize().height;
				
		int m=w;
		if (h>m) m=h;
		g.fillRect(0,0,m,m);
		endOfCode=0;    		
   	//tdw and BRJ
		// draw bars
		currentX = leftMarginPixels;
		if (barType==CODE11)  paintCODE11(g,false);
		if (barType==MSI)  paintMSI(g,false);
		if (barType==CODABAR)  paintCODABAR(g,false);
		if (barType==CODE39)  paintCODE39(g,false);
		if (barType==CODE39EXT) paintCODE39Ext(g,false);
		if (barType==INTERLEAVED25) paintInterleaved25(g,false);
		if (barType==CODE93) paintBAR93(g,false);
		if (barType==EAN8) paintEAN8(g,false);
		if (barType==EAN13) paintEAN13(g,false);
		if (barType==UPCA) paintUPCA(g,false);
		if (barType==UPCE) paintUPCE(g,false);
		if (barType==CODE128) paintCode128(g,false);
		if (barType==CODE93EXT) paintBAR93Ext(g,false);
		if (barType==IND25) paintIND25(g,false);
		if (barType==MAT25) paintMAT25(g,false);
		if (barType==POSTNET) paintPOSTNET(g,false);
		if (barType==PLANET) paintPLANET(g,false);
		//if (barType==UCC128) paintUCC128(g,false);
		if (barType==UCC128) paintCode128(g,false);
		if (barType==ONECODE) paintONECODE(g,false);
		if (endOfCode==0) endOfCode=currentX;
		//leftMarginPixels = 0;
	  //gets the size of the barcode.
	  
	  L = ((currentX- leftMarginPixels)/resolution);	  	 
	  		
		if (codeText.length()==0)  codeText=code;	  
		

		//starting point of demoPixelW brj 06/06/06 
	  int demoX = 0;
		//we must set font size back to powered by idautmation size of 11
		g.setFont(new Font("Arial",Font.PLAIN,11));
	  demoPixelW = g.getFontMetrics().stringWidth("Powered by IDAutomation.com");
	  
	  g.setFont(new Font(textFont.getName(),textFont.getStyle(),(int)(textFont.getSize()*resolution*.027)));//tb
		
 		//width of CodeText after font sizing
		codeTextW = g.getFontMetrics().stringWidth(codeText);		
	
		if (barType==ONECODE)codeTextW+=leftMarginPixels;
    //brj 7/18
    // int bcw = ((int)(L*resolution));
    int bcw = (currentX -leftMarginPixels);
    bcw = bcw + 2* leftMarginPixels;
    bcw = bcw + (int) (2* (fBearerBarVert * narrowBarPixels));

    int hrLeftMarginPixels = 0;
		if(!showText)
			codeTextW = 0;
 		//width of CodeText after font sizing
 		int modVar =0;
 		if (ValSpliceText >0)
 		  modVar = (codeText.length() / ValSpliceText)-1;
 		  

		codeTextW += (g.getFontMetrics().stringWidth(" ") * modVar) ;


		if (barType==ONECODE) {
			int widthOfSpace = g.getFontMetrics().stringWidth(" ");
			if (codeText.length()==31) codeTextW+=(6*widthOfSpace);
			if (codeText.length()==29) codeTextW+=(5*widthOfSpace);
			if (codeText.length()==25) codeTextW+=(4*widthOfSpace);
			if (codeText.length()==20) codeTextW+=(3*widthOfSpace);
		}		
	  // leftMarginPixels = origLeftMarginPixels;
	
	
	//In this section of code, we define pWidth.  We set pWidth to either codeTextW (human readable text), demoPixelW (demo text), or bcw+ 2 *leftMarginPixels
	//(barcode with margins) //brj 8/17/06   
	 
	 if((codeTextW >= demoPixelW) && (codeTextW >= bcw))
     	{ 
     		pWidth = codeTextW;
     		
     	 }
     	 
   if((demoPixelW >= codeTextW) && (demoPixelW >= bcw))
     	{ 
     		pWidth = demoPixelW;
     	} 	 
	
	 if((bcw >= codeTextW) && (bcw>= demoPixelW))
     	{  
     		pWidth = bcw;
     	} 
		//retrieves the center point of barcode
		currentX = pWidth/2 - bcw/2 +leftMarginPixels + (int)(fBearerBarVert * narrowBarPixels) ;
		if (barType==ONECODE)currentX=leftMarginPixels;
  	//retrieves the center point of human readable
		hrLeftMarginPixels = (pWidth/2 - codeTextW/2); 
		
		//retrieves the center point of the demo text
		demoX = (pWidth/2 - demoPixelW/2);

  
           
 //**************************************************************************************************************************    
		
	// brj text margin
		textMarginPixels = (int) (textMarginCM * resolution);
		 
		
		//set font to demo font size to set the height of the demo text
		g.setFont(new Font("Arial",Font.PLAIN,11));
		
		
		int DemoPixelH = g.getFontMetrics().getHeight();
		//switch back to original font
		g.setFont(new Font(textFont.getName(),textFont.getStyle(),(int)(textFont.getSize()*resolution*.027)));//tb

		if ((showText) && (textFont!=null)) {
			g.setFont(new Font(textFont.getName(),textFont.getStyle(),(int)(textFont.getSize()*resolution*.027)));//tb
			TextH=g.getFontMetrics().getHeight();
			if ((rotate==0) || (rotate==180)){
				if (d==1) {
				
						pHeight = (int) (barHeightPixels+TextH+(topMarginPixels*2)+textMarginPixels)+2*(int) (fBearerBarHori * narrowBarPixels);	
					if ((barType==POSTNET) || (barType==PLANET)) 
						pHeight =(int)(this.postnetHeightTallBar*resolution)+TextH+(topMarginPixels*2)+textMarginPixels;
				}
				if (d!=1){
											
					pHeight = (int) (barHeightPixels+TextH+(topMarginPixels*2)+textMarginPixels+DemoPixelH+2*(int) (fBearerBarHori * narrowBarPixels));	
					if ((barType==POSTNET) || (barType==PLANET)) 
						pHeight =(int)(this.postnetHeightTallBar*resolution)+TextH+DemoPixelH+(topMarginPixels*2)+textMarginPixels;	
				}
			}

			else{	//rotate = 90 or 270
				if (d==1){
         
          pHeight = (int) (barHeightPixels+TextH+(topMarginPixels*2)+textMarginPixels)+2*(int) (fBearerBarHori * narrowBarPixels);
					if ((barType==POSTNET) || (barType==PLANET))
						pHeight =(int)(this.postnetHeightTallBar*resolution)+TextH+(topMarginPixels*2)+textMarginPixels;
				}
				if(d!=1){
					pHeight = (int) (barHeightPixels+TextH+(topMarginPixels*2)+textMarginPixels+DemoPixelH+2*(int) (fBearerBarHori * narrowBarPixels));
					if ((barType==POSTNET) || (barType==PLANET))
						pHeight =(int)(this.postnetHeightTallBar*resolution)+TextH+DemoPixelH+(topMarginPixels*2)+textMarginPixels;
				}
			}
		}
		else {	//showText = false
			if ((rotate==0) || (rotate==180)){
				if (d==1) {
				
					pHeight = (int) (barHeightPixels+(topMarginPixels*2)+textMarginPixels+ 2*(int) (fBearerBarHori * narrowBarPixels));///ben added recently testing purposes
					
					if ((barType==POSTNET) || (barType==PLANET))
						pHeight =(int)(this.postnetHeightTallBar*resolution)+1+(topMarginPixels*2);	
				}
				if (d!=1){
					
						if (barType == EAN8 || barType == EAN13 || barType == UPCE || barType == UPCA)
					     {
					     	pHeight = (int) (barHeightPixels+(topMarginPixels*2)+DemoPixelH+textMarginPixels+2*(int) (fBearerBarHori * narrowBarPixels));
					     }
					
					else {
					
					pHeight = (int) (barHeightPixels+(topMarginPixels*2)+DemoPixelH+textMarginPixels+2*(int) (fBearerBarHori * narrowBarPixels));		
					  }		
					if ((barType==POSTNET) || (barType==PLANET))
						pHeight =(int)(this.postnetHeightTallBar*resolution)+DemoPixelH+5+(topMarginPixels*2);	
				}
			}
			else{	//rotate = 90 or 270		
				if(d==1){
			
				pHeight = (int) (barHeightPixels+(topMarginPixels*2)+textMarginPixels+ 2*(int) (fBearerBarHori * narrowBarPixels));///ben added recently testing purposes
					if ((barType==POSTNET) || (barType==PLANET))		
						pHeight =(int)(this.postnetHeightTallBar*resolution)+1+(topMarginPixels*2);	
				}			    
				if(d!=1){
					
					if (barType == EAN8 || barType == EAN13 || barType == UPCE || barType == UPCA)
					     {
					     	pHeight = (int) (barHeightPixels+(topMarginPixels*2)+DemoPixelH+textMarginPixels+2*(int) (fBearerBarHori * narrowBarPixels));
					     }
					
				
           else {
					pHeight = (int) (barHeightPixels+(topMarginPixels*2)+DemoPixelH+textMarginPixels+2*(int) (fBearerBarHori * narrowBarPixels));
				      }
					if ((barType==POSTNET) || (barType==PLANET))		
						pHeight =(int)(this.postnetHeightTallBar*resolution)+DemoPixelH+5+(topMarginPixels*2);
				}
			}
		}	

		//variable ybuf = 5 adds spacing between the human readable and demo text
		int ybuf = 7;
		
		//starting point of first character for EAN13 and UPCA symbologies. brj 
		int startFlag = currentX;
/*
		//Added by TB 2/2006, increase the size of the canvas to include the thickness of the vertical bearer bars
   	if(rotate == 0 || rotate == 180)//increase width
    	pWidth += 2 * ((int)(fBearerBarVert * narrowBarPixels));
    if(rotate == 90 || rotate == 270)//increase height
     	pHeight += 2 * ((int)(fBearerBarVert * narrowBarPixels));
    */   

		// draw bars
		if (barType==CODE11)  paintCODE11(g,true);
		if (barType==MSI)  paintMSI(g,true);
		if (barType==CODABAR)  paintCODABAR(g,true);
		if (barType==CODE39)  paintCODE39(g,true);
		if (barType==CODE39EXT) paintCODE39Ext(g,true);
		if (barType==INTERLEAVED25) paintInterleaved25(g,true);
		if (barType==CODE93) paintBAR93(g,true);
		if (barType==EAN8) paintEAN8(g,true);
		if (barType==EAN13) paintEAN13(g,true);
		if (barType==UPCA) paintUPCA(g,true);
		if (barType==UPCE) paintUPCE(g,true);
		if (barType==CODE128) paintCode128(g,true);
		if (barType==CODE93EXT) paintBAR93Ext(g,true);
		if (barType==IND25) paintIND25(g,true);
		if (barType==MAT25) paintMAT25(g,true);
		if (barType==POSTNET) paintPOSTNET(g,true);
		if (barType==PLANET) paintPLANET(g,true);
		//if (barType==UCC128) paintUCC128(g,true);
		if (barType==UCC128) paintCode128(g,true);
		if (barType==ONECODE) paintONECODE(g,true);
		if (endOfCode==0) endOfCode=currentX;
		currentY=barHeightPixels+topMarginPixels;
		if (autoSize) {
			
			if (rotate==0||rotate==180) 
			  this.setSize(pWidth,pHeight);
			else this.setSize(pHeight,pWidth);
			
	  }
		if(barType == POSTNET || barType == PLANET)
    	currentY=(int)(postnetHeightTallBar*resolution)+1+topMarginPixels;
    	
    g.setFont(new Font(textFont.getName(),textFont.getStyle(),(int)(textFont.getSize()*resolution*.027)));//tb
		TextH=g.getFontMetrics().getHeight();
				
		//width of CodeText after font sizing
		codeTextW = g.getFontMetrics().stringWidth(codeText);
		
		//g.setFont(new Font("Arial",Font.PLAIN,11));
 		DemoPixelH = g.getFontMetrics().getHeight();
		//ybuf+= DemoPixelH;
		ybuf+= TextH;
	    // draw text
		if ((showText) && (textFont!=null)) {
	
			g.setColor(fontColor);
			//g.setFont(new Font(textFont.getName(),textFont.getStyle(),(int)(textFont.getSize()*resolution*.027)));//tb
			TextH=g.getFontMetrics().getHeight();
			int charW=g.getFontMetrics().stringWidth("X");
				
			// draw supplement text
			if ((this.UPCEANSupplement2) || (this.UPCEANSupplement5))
			 if ((barType==EAN8)||(barType==UPCA)||(barType==UPCE)||(barType==EAN13)) {
				
			   int groupCenterX=(int) ((endSuplement-startSuplement-g.getFontMetrics().stringWidth(codeSup))/2);
			   if (groupCenterX<0) groupCenterX=0;
			   	//DemoDraw(g,demoX, ybuf,TextH);
			   g.drawString(codeSup,startSuplement+groupCenterX,topMarginPixels+suplementTopMargin-2);
			}
						
		  //BRJ  increases the height of the human readable from barcode for EAN8,EAN13,UPCA,UPCE  6/28/2006
			int increaseheight = 0;	 
              	
			//Added by TB 6/2006, splice text for human readable
			if (ValSpliceText == 3)
				codeText = spliceText(codeText,3);
			if (ValSpliceText == 4)
				codeText = spliceText(codeText,4);
			if (ValSpliceText == 5)
				codeText = spliceText(codeText,5);

      if ((barType==ONECODE) )  {
			  int toCenterX=0;// we want the text to start at 0...(int) ((endOfCode-leftMarginPixels-g.getFontMetrics().stringWidth(codeText))/2);
			  if (toCenterX<0) toCenterX=0;
			  hrLeftMarginPixels=leftMarginPixels;	  
			  g.drawString(codeText,hrLeftMarginPixels,(int) ((int)(.5*TextH)+(this.postnetHeightShortBar*resolution*3)+(int)textMarginPixels+topMarginPixels));//tb 8/11/2006
			
			  currentY=(int)(this.postnetHeightTallBar*resolution)+TextH+1+topMarginPixels;
			  
			  //Powered by IDAutomation			  
				DemoDraw(g,demoX, ybuf +5,TextH);
				DrawBearerBars(g,0,bcw);
			  return;
		    }  
			// draw text for POSTNET
			if ((barType==POSTNET) || (barType==PLANET))  {
			  int toCenterX=(int) ((endOfCode-leftMarginPixels-g.getFontMetrics().stringWidth(codeText))/2);
			  if (toCenterX<0) toCenterX=0;
			  g.drawString(codeText,hrLeftMarginPixels,(int) ((this.postnetHeightTallBar*resolution)+(int)(.5*TextH)+textMarginPixels+topMarginPixels));//tb 8/11/2006
			
			  currentY=(int)(this.postnetHeightTallBar*resolution)+TextH+1+topMarginPixels;
			  
			  //Powered by IDAutomation			  
				DemoDraw(g,demoX, ybuf,TextH);
				DrawBearerBars(g,0,bcw);
			  return;
		    }  
		    
/////////////////////////////////////////////EAN13///////////////////////////////////////////////////
		    // draw text for EAN 13
		    if ((barType==EAN13)&& (guardBars) && (codeText.length()>=13)) {

			  int groupCenterX=0;
				
		if (codeText == "Incorrect Data To Encode")
		 {
		 g.drawString(codeText,hrLeftMarginPixels,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
	   }
	
	else {
		
					
			  // first flag
				
				g.drawString(codeText.substring(0,1),startFlag-charW,barHeightPixels+textMarginPixels+topMarginPixels);        				                   
			  groupCenterX=(int) (((centerGuardBarStart-leftGuardBar)-g.getFontMetrics().stringWidth(codeText.substring(1,7)))/2);
			  if (groupCenterX<0) groupCenterX=0;

			  // next 6 chars
			 
			    g.drawString(codeText.substring(1,7),leftGuardBar+groupCenterX,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels); //brj 6/28/2006
					         
			  groupCenterX=(int) (((rightGuardBar-centerGuardBarEnd)-g.getFontMetrics().stringWidth(codeText.substring(7,13)))/2);
			  if (groupCenterX<0) groupCenterX=0;

			  // last 6 chars
			   g.drawString(codeText.substring(7,13),centerGuardBarEnd+groupCenterX,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels); //brj 6/28/2006
       
			  currentY=barHeightPixels+TextH+1+topMarginPixels;
			}   
			   //Powered by IDAutomation
			   DemoDraw(g,demoX, ybuf,TextH); 
			   DrawBearerBars(g,0,bcw);
			  return;
		    }

/////////////////////////////////////////////UPCA//////////////////////////////////////////////////////////
		    // draw text for UPCA
		    if ((barType==UPCA) && (guardBars) && (codeText.length()>=12)) {

			  int groupCenterX=0;

				     	if (codeText == "Incorrect Data To Encode")
		 {
		 g.drawString(codeText,hrLeftMarginPixels,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
	   }             
				else {
				g.drawString(codeText.substring(0,1),startFlag-charW,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels); //tb 7/11/2006  
				
					      					      
			  groupCenterX=(int) (((centerGuardBarStart-leftGuardBar)-g.getFontMetrics().stringWidth(codeText.substring(1,6)))/2);
			  if (groupCenterX<0) groupCenterX=0;

			  // next 5 chars		 
       	g.drawString(codeText.substring(1,6),leftGuardBar+groupCenterX,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
          
			  groupCenterX=(int) (((rightGuardBar-centerGuardBarEnd)-g.getFontMetrics().stringWidth(codeText.substring(6,11)))/2);
			  if (groupCenterX<0) groupCenterX=0;

			  // last 5 chars 
       	g.drawString(codeText.substring(6,11),centerGuardBarEnd+groupCenterX,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
          
			  // last char			
			 	g.drawString(codeText.substring(11,12),endOfCode+3,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
			    
			  currentY=barHeightPixels+TextH+1+topMarginPixels;
			}
			   //Powered by IDAutomation
				 DemoDraw(g,demoX, ybuf,TextH);
				 DrawBearerBars(g,0,bcw);
			  return;
		    }

/////////////////////////////////////////////EAN8////////////////////////////////////////////////////////////////
		    // draw text for EAN 8
		    if ((barType==EAN8)&& (guardBars) && (codeText.length()>=8)) {

			  int groupCenterX=0;
			  
			  	if (codeText == "Incorrect Data To Encode")
		 		{
			  g.drawString(codeText,hrLeftMarginPixels,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
	   		}
			  
       else {
			  groupCenterX=(int) (((centerGuardBarStart-leftGuardBar)-g.getFontMetrics().stringWidth(codeText.substring(0,4)))/2);
			  if (groupCenterX<0) groupCenterX=0;

			  // first 4 chars		 
        g.drawString(codeText.substring(0,4),leftGuardBar+groupCenterX,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
        
			  groupCenterX=(int) (((rightGuardBar-centerGuardBarEnd)-g.getFontMetrics().stringWidth(codeText.substring(4,8)))/2);
			  if (groupCenterX<0) groupCenterX=0;

			  // last 4 chars			 
			  g.drawString(codeText.substring(4,8),centerGuardBarEnd+groupCenterX,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
			  
			  currentY=barHeightPixels+TextH+1+topMarginPixels;
			}
			  //Powered by IDAutomation
				DemoDraw(g,demoX, ybuf,TextH);
				DrawBearerBars(g,0,bcw);
			  return;
		    }

/////////////////////////////////////////////UPCE//////////////////////////////////////////////////////////
		    // draw text for UPCE /////////
		     if ((barType==UPCE)&& (guardBars)&& (codeText.length()>=8)) {

			  int groupCenterX=0;

    			if (codeText == "Incorrect Data To Encode")
		 			{
		 			g.drawString(codeText,hrLeftMarginPixels,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
	  			 }

        else {
			  // first char
        g.drawString(codeText.substring(0,1),startFlag-charW,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);//tb 7/24/06
				
			  groupCenterX=(int) (((rightGuardBar+2-leftGuardBar)-g.getFontMetrics().stringWidth(codeText.substring(1,7)))/2);
			  if (groupCenterX<0) groupCenterX=0;

			  // next 6 chars			
       	g.drawString(codeText.substring(1,7),leftGuardBar+groupCenterX,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
          
      	//---------------- last char (added!!) ----------------------			  
      	g.drawString(codeText.substring(7,8),endOfCode+2,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
            
			  currentY=barHeightPixels+TextH+1+topMarginPixels;
         }
         //Powered by IDAutomation
				DemoDraw(g,demoX, ybuf,TextH);
				DrawBearerBars(g,0,bcw);
			  return;
		    }
	 
	 ////////////////////////////////////////////////other symbologies/////////////////////////////////////////////
						
			int toCenterX=(int) ((endOfCode-leftMarginPixels-g.getFontMetrics().stringWidth(codeText))/2);
	   	  
	   	  if (codeText == "Incorrect Data To Encode")
		 			{
		 			g.drawString(codeText,hrLeftMarginPixels,barHeightPixels+textMarginPixels+increaseheight+topMarginPixels);
	  			 }
	   	else {    
	   	if (toCenterX<0) toCenterX=0;
	   	 		  		     	     	
			g.drawString(codeText,hrLeftMarginPixels,barHeightPixels+(int)(.5*TextH)+textMarginPixels+topMarginPixels +((int)(fBearerBarHori *narrowBarPixels)));							         		
			currentY=barHeightPixels+TextH+1+topMarginPixels+((int)(fBearerBarHori *narrowBarPixels));
		}
		    }
	//////////////////////////////////////////////////////DRAW TEXT ////////////////////////////////////////////
				DemoDraw(g,demoX,ybuf+((int)(fBearerBarHori *narrowBarPixels)),TextH);
				DrawBearerBars(g,0,bcw);	
	}
	//////////////////////////////////END OF PAINT BASIS////////////////////////////////////////////////////						
							
							
   // function to rotate an area of an image, 90, 180 or 270 degrees
		/** rotates an image */	

   protected Image rotate(Image im, int angle,int maxw, int maxh) {
        int w=im.getWidth(null);
        int h= im.getHeight(null);
		
		if (maxw > w ) maxw=w;
		if (maxh > h ) maxh=h;
		
        int[] pixels = new int[w * h];
        int[] pixels2 = new int[maxw * maxh];
	PixelGrabber pg = new PixelGrabber(im, 0,0, w, h, pixels, 0, w);
	try {
	    pg.grabPixels();
	} catch (InterruptedException e) {
	    System.err.println("interrupted waiting for pixels!");
	    return null;
	}

	if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
	    System.err.println("image fetch aborted or errored");
	    return null;
	}

     if (angle==90) {
   	    for (int i = 0; i < maxw; i++) {
	      	for (int j = 0; j < maxh; j++) {
		   			pixels2[(maxh*(maxw-(i+1)))+j]=pixels[(j * w) + i];
	      	}
	     	}
        return Toolkit.getDefaultToolkit().createImage(new MemoryImageSource( maxh,maxw,pixels2,0,maxh));
     	}
    
     	if (angle==180) {
   	 		for (int i = 0; i < maxw; i++) {
	      	for (int j = 0; j < maxh; j++) {
		     pixels2[((maxh-(j+1)) * maxw) + (maxw-(i+1))]=pixels[(j * w) + i];
	      }
	  	}	
       return Toolkit.getDefaultToolkit().createImage( new MemoryImageSource(maxw,maxh,pixels2,0,maxw));
      }
			
      if (angle==270) {
   	    for (int i = 0; i < maxw; i++) {
	      	for (int j = 0; j < maxh; j++) {
		   			pixels2[(maxh*i)+(maxh-(j+1))]=pixels[(j * w) + i];
	      	}
	     	}
        return Toolkit.getDefaultToolkit().createImage( new MemoryImageSource( maxh,maxw,pixels2,0,maxh));
      }
       return null;
   }
   
	//Added by TB 2/2006, splice text for human readable
	protected String spliceText (String TextToSplice, int SpcacingNumber)
	{
		StringBuffer HumanReadableText = new StringBuffer("");
		int j = 0;

		for(int i = 0; i < TextToSplice.length(); i++)
		{
			char CurrentCharNum = TextToSplice.charAt(i);
			if (CurrentCharNum > 31 && CurrentCharNum < 128)
			{
				HumanReadableText = HumanReadableText.append( TextToSplice.charAt(i));
				j++;
			}
	
			if ((j % SpcacingNumber) == 0)
			{
				HumanReadableText = HumanReadableText.append( " " );
			}
		}
		return HumanReadableText.toString();
		
	}
	
	
	
	//Added by TB 2/2006, draws bearer bars around barcode
	protected void DrawBearerBars(Graphics g,int bartopmargin, int bcw)
	{
		
	
		 //Bearer Bar Horizontal disabled for POSTNET,PLANET, and ONE CODE
			if (barType ==POSTNET || barType ==PLANET || barType == ONECODE) {
		   fBearerBarHori = 0;
		   fBearerBarVert = 0;
			PropBearerBarHori = 0;
			PropBearerBarVert = 0;
		}
	
	
	
		
		int vertBearerBarThickness = (int) (fBearerBarVert * narrowBarPixels);
		int horiBearerBarThickness = (int) (fBearerBarHori * narrowBarPixels);
		int offSet = 0; //variable for human readable for UPC, EAN
		int horizontalEdge = currentX+leftMarginPixels + vertBearerBarThickness;		
		g.setColor(barColor);
		
		if (barType == EAN8 || barType == EAN13 || barType == UPCE || barType == UPCA)
				offSet = g.getFontMetrics().getHeight();
		/*	
		if (barType==POSTNET || barType==PLANET || barType==ONECODE)
		{
		 vertBearerBarThickness = 0;
		 horiBearerBarThickness = 0;
		}
*/

		if(fBearerBarVert > 0)

			{	//increase the left margin to 10 times the X dimension
			  //vertical bearer bars are in use.
			if (leftMarginCM < (10 * X))
				leftMarginCM = 10 * X;

		


			//right bar
			g.fillRect(currentX+leftMarginPixels,topMarginPixels,vertBearerBarThickness,barHeightPixels+extraHeight+offSet);
		
		     
			//left bar
			g.fillRect(currentX+leftMarginPixels + vertBearerBarThickness -bcw ,topMarginPixels,vertBearerBarThickness,barHeightPixels+extraHeight+offSet);  //orignal
		
		}
		 
		if(fBearerBarHori > 0)
		{
			//top bar
				g.fillRect(currentX+leftMarginPixels + vertBearerBarThickness -bcw,topMarginPixels, bcw, horiBearerBarThickness);
			
			//bottom bar
			
		  g.fillRect(currentX+leftMarginPixels+vertBearerBarThickness-bcw, topMarginPixels+barHeightPixels+offSet, bcw, horiBearerBarThickness);
		
		}
	}
	
	
	
    // BRJ 6/22/2006 Adds Powered by IDAutomation underneath human readable.  If show Text false, adds it underneath barcode.	
		protected void DemoDraw(Graphics g, int demoX, int ybuf, int TextH){
		
			if (d!=1) {
				g.setFont(new Font("Arial",Font.PLAIN,11));
					
				g.setColor(Color.decode("#999999")); 
				
		 		//if POSTNET or PLANET DEMO  Position of demo text is calculated 
     		if ((barType==POSTNET) || (barType==PLANET)) {
     		// if show text is true for PLANET or POSTNET    
     			if (showText) {
		    		g.drawString("Powered by IDAutomation.com",demoX,(int)(this.postnetHeightTallBar*resolution)+ybuf+textMarginPixels+topMarginPixels);
		      }
		 			//show text false, human readable is elminated from POSTNET or PLANET
		 			else {
		    		g.drawString("Powered by IDAutomation.com",demoX,(int)(this.postnetHeightTallBar*resolution)+ybuf+topMarginPixels);
		      }
		    }
		   
		   //*****************If any other SYMBOLOGY**********************///      BRG 6/28/2006
		  	else {  
				// if show text is true. 
					if (showText){
						g.drawString("Powered by IDAutomation.com",demoX,barHeightPixels+textMarginPixels+ybuf+topMarginPixels);
			  	}
			          
					//show Text false, human readable is eliminated.
    			else {
						
					if (barType == EAN8 || barType == EAN13 || barType == UPCE || barType == UPCA){ 
						g.drawString("Powered by IDAutomation.com",demoX,barHeightPixels+textMarginPixels+ybuf+topMarginPixels);//tb 7/28/2006
						}
						else {
						
						
						g.drawString("Powered by IDAutomation.com",demoX,barHeightPixels+textMarginPixels-TextH+ybuf+topMarginPixels);//tb 7/28/2006
					}
				  }
				}	
			
			}//end of if demo
				
			//sets font size and color to user settings
	    g.setFont(new Font(textFont.getName(),textFont.getStyle(),(int)(textFont.getSize()*resolution*.027)));//tb
	    g.setColor(fontColor);
	    }//end of draw demo
	    
	    
	    
	    
	    
	    
static boolean MultiplyBytesByShort( int[] byteArray,int NumberOfBytes,int  Multiplicand  )

		{
			int         ByteIndex;
			int  Carry32,Temp32;

			/* Check for obviously incorrect inputs */
			if ( byteArray == null )
				return false;
			if ( NumberOfBytes < 1 )
				return false;

			/* Do groups of two bytes */
			Carry32 = 0;

			for ( ByteIndex = NumberOfBytes-1; ByteIndex > 0; ByteIndex -= 2 )
			{
				/* Fill two low bytes of four byte variable with packed data */
				Temp32  = byteArray[ByteIndex];
				Temp32 |= (byteArray[ByteIndex-1] << 8);
				
				//System.out.println("the BA-1 ="+(byteArray[ByteIndex-1] << 8));
				/* 0x0000???? * 0x0000???? = 0xCCCCRRRR (C-carry data, R-result data) */
				Temp32 *= Multiplicand;
				Temp32 += Carry32;

				/* The two low bytes (result) go back into the packed data */
				byteArray[ByteIndex  ] = Temp32 & 0x000000FF;
				byteArray[ByteIndex-1] = (Temp32 >> 8) & 0x000000FF;


				/* The two high bytes will be the carry data for the next pass */
				Carry32 = (Temp32 >> 16);

			}

			/* Single byte left over? */
			if ( ByteIndex == 0 )
			{
				Temp32  = byteArray[0];
				Temp32 *= Multiplicand;
				Temp32 += Carry32;

				/* The low byte goes back into the packed data */
				byteArray[0] = ((int)Temp32 & 0xFF) ;
			}

			return true;
		}
		static boolean AddShortToBytes( int[]  ByteArrayPtr, int NumberOfBytes, int  Addend)

		{
			int         ByteIndex;
			int  Carry32,Temp32;

			/* Check for obviously incorrect inputs */
			if ( ByteArrayPtr == null )
				return false;
			if ( NumberOfBytes < 1 )
				return false;

			/* Fill two low bytes of four byte variable with packed data */
			Temp32  = (int)ByteArrayPtr[NumberOfBytes-1];

			Temp32 |= (int)ByteArrayPtr[NumberOfBytes-2] << 8;
			/* Add a two byte value into the four byte variable */
			Temp32 += (int)Addend;

			/* The two low bytes go back into the packed area */
			ByteArrayPtr[NumberOfBytes-1] = Temp32 & 0x000000FF;
			ByteArrayPtr[NumberOfBytes-2] = (Temp32 >> 8) & 0x000000FF;

			/* A single bit carry may be generated */
		
			Carry32 = Temp32 | 0xFFFF;

			/* Propagate carry up */
			for (ByteIndex = NumberOfBytes-3;(Carry32 == 1)&&(ByteIndex > 0);ByteIndex--)
			{
				Temp32 = Carry32 +  ByteArrayPtr[ByteIndex];

				ByteArrayPtr[ByteIndex] = Temp32 & 0x000000FF;
				Carry32 =(Temp32 | 0xFF);
			}

			return true;
		}

		static int GenerateCRC11FrameCheckSequence( int[]  ByteArrayPtr )

		{
			int  GeneratorPolynomial = 0x0F35;
			int  FrameCheckSequence  = 0x07FF;
			int  Data;
			int     ByteIndex,Bit;


			/* Do most significant byte skipping most significant bit */
			Data = (int)(ByteArrayPtr[0] << 5);
			//ByteArrayPtr++;
			for ( Bit = 2; Bit < 8; Bit++ )
			{
				if ( ((FrameCheckSequence ^ Data) & 0x400 )!=0)
					FrameCheckSequence = (int)((FrameCheckSequence << 1) ^ GeneratorPolynomial);
				else
					FrameCheckSequence = (int)(FrameCheckSequence << 1);
				FrameCheckSequence &= 0x7FF;
				Data <<= 1;
			}
			/* Do rest of the bytes */
			for ( ByteIndex = 1; ByteIndex < 13; ByteIndex++ )
			{
				Data = (int)(ByteArrayPtr[ByteIndex] << 3);
				//ByteArrayPtr++;
				for ( Bit = 0; Bit < 8; Bit++ )
				{
					if ( ((FrameCheckSequence ^ Data) & 0x0400)!=0 )
						FrameCheckSequence = (int)((FrameCheckSequence << 1) ^ GeneratorPolynomial);
					else
						FrameCheckSequence = (int)(FrameCheckSequence << 1);
					FrameCheckSequence &= 0x7FF;
					Data <<= 1;
				}
			}

			return FrameCheckSequence;
		}
		private boolean GetNof13Table( int N, int[] TableArrayPtrPtr )
		{
			switch ( N )
			{
				case 2:
					if ( Table2of13InitializedFlag == false )
						if ( InitializeNof13Table( Table2of13, 2, TABLE_2_OF_13_SIZE) != true )
							return false;
					TableArrayPtrPtr     = Table2of13;
					NumberOf2of13TableEntries = TABLE_2_OF_13_SIZE;
					Table2of13InitializedFlag = true;
					return true;
				case 5:
					if ( Table5of13InitializedFlag == false )
						if ( InitializeNof13Table( Table5of13, 5, TABLE_5_OF_13_SIZE) != true )
							return false;
					TableArrayPtrPtr     = Table5of13;
					NumberOf5of13TableEntries = TABLE_5_OF_13_SIZE;
					Table5of13InitializedFlag = true;
					return true;
				default:
					return false;
			}
		}
		static boolean InitializeNof13Table( int[] TableNof13,int  N,int  TableLength )

		{
			int      Count,Reverse;
			int      LUT_LowerIndex,LUT_UpperIndex;
			int      BitCount;
			int      BitIndex;
			boolean  SymmetricFlag;


			/* Count up to 2^13 and find all those values that have N bits on */
			LUT_LowerIndex = 0;
			LUT_UpperIndex = TableLength - 1;

			for ( Count = 0; Count < 8192; Count++ )
			{
				BitCount = 0;
				for ( BitIndex = 0; BitIndex < 13; BitIndex++ )
					if ((Count & (1 << BitIndex)) != 0)
						BitCount ++; //((Count & (1 << BitIndex)) );

				/* If we don't have the right number of bits on, go on to the next value */
				if ( BitCount != N )
					continue;

				Reverse = ReverseShort( Count ) >> 3;

				SymmetricFlag = Count == Reverse;

				/* If the reverse is less than count, we have already visited this pair before */
				if ( Reverse < Count )
				{
					continue;
				}

				if ( SymmetricFlag ==  true )
				{
					TableNof13[LUT_UpperIndex] = Count;
					LUT_UpperIndex -= 1;
				}
				else
				{
					TableNof13[LUT_LowerIndex] = Count;
					LUT_LowerIndex += 1;
					TableNof13[LUT_LowerIndex] = Reverse;
					LUT_LowerIndex += 1;
				}
			}

			/* We better have the exact correct number of table entries */
			if ( LUT_LowerIndex != (LUT_UpperIndex+1) )
				return false;

			return true;
		}


		static int ReverseShort( int  Input )
		{
			int  Reverse = 0;
			int             Index;


			for ( Index = 0; Index < 16; Index++ )
			{
				Reverse <<= 1;
				Reverse  |= (int)(Input & 1);
				Input   >>= 1;
			}

			return Reverse;
		}
		private boolean ConvertFromBytesToMultiBase(int NumberOfBytes,int NumberOfNumbers )
		{
			int     NumberIndex;
			for ( NumberIndex = NumberOfNumbers-1; NumberIndex >= 0; NumberIndex-- )
			{
				if ( DivideBytesByShort(NumberOfBytes,(int)CodewordArray[NumberIndex][0]) != true)
					return false;
				CodewordArray[NumberIndex][1] = Remainder;
			}
			return true;
		}
		private boolean DivideBytesByShort(int NumberOfBytes,int Divisor)
		{
			int ByteIndex;
			int Temp32,Remainder32;


			/* Check for obviously incorrect inputs */
			if ( byteArray == null )
				return false;
			if ( NumberOfBytes < 2 )
			{
				return false;
			}
			if ( Divisor == 0 )
			{
				return false;
			}
			/* If we do not have an even number of bytes, do the first byte separately */
			if ( (NumberOfBytes % 2) == 1 )
			{
				Temp32 = (int)byteArray[0];
				Remainder32  = Temp32 % Divisor;
				Temp32      /= Divisor;

				byteArray[0] = (byte)Temp32;
				ByteIndex = 1;
			}
			else
			{
				Remainder32 = 0;
				ByteIndex   = 0;
			}

			/* Now that we have an even number of bytes left, go from left to right in
			groups of two */
			for ( ; ByteIndex < NumberOfBytes; ByteIndex += 2 )
			{
				/* Build up a slice consisting of the previous slices remainder in the
					top 16 and data in the low 16 */
				Temp32  = Remainder32 << 16;
				Temp32 |= (int)byteArray[ByteIndex] << 8;
				Temp32 |= (int)byteArray[ByteIndex+1];

				Remainder32  = Temp32 % Divisor;
				Temp32      /= Divisor;

				/* Replace slice of dividend with slice of quotient */
				byteArray[ByteIndex  ] = (Temp32 >> 8);
				byteArray[ByteIndex+1] = Temp32;
			}

			Remainder = (short) Remainder32;

			return true;
		}
///<summary>Wrapper for the USPS OneCode Symbology</summary>
protected void paintONECODE(Graphics g ,boolean paint)
		{
			setXDimensionCM(.06);
	   

			StringBuffer UserDataEncoded=new StringBuffer("");			
			Table2of13InitializedFlag = false;
			Table5of13InitializedFlag = false;
			NumberOf5of13TableEntries=0L; 
			NumberOf2of13TableEntries=0L; 
			Remainder=0;
			Table2of13 = new int[TABLE_2_OF_13_SIZE];
			Table5of13= new int[TABLE_5_OF_13_SIZE];
			byteArray = new int[13];
			int[] BarTopArray= new int[65];
			int[] BarBottomArray=new int[65];
			//NumberRecordType[]  CharacterArray = new NumberRecordType[10];
			double[][] CharacterArray = new double[10][2];
			int    FrameCheckSequence11BitValue=0;
			StringBuffer returnStr = new StringBuffer();
			// We test for a comma in case the USPS changes their spec later on.  
			// This will allow us a delimiter if they ever change the tracking 
			// length to something other then 20
			StringBuffer strRoute = new StringBuffer();
			StringBuffer strTrack = new StringBuffer();
      if (code.length()<20){
      	 codeText="Incorrectly formatted DataToEncode";
         return;
      }
			strTrack.append(code.substring(0,20));
			strRoute.append(code.substring(20));
			if (strTrack.length()!=20)
			{
				System.out.println("Incorrectly formatted tracking number");
				codeText="Incorrectly formed DataToEncode parameter";				
				return;
			}
			for (int i=0;i<strRoute.length();i++) 
			{
				if (!Character.isDigit(strRoute.charAt(i)) )
				{
					System.out.println("Incorrectly formatted Routing number");
					codeText="Incorrectly formed DataToEncode parameter";							
					return;
				}
			}
			for (int i=0;i<strTrack.length();i++) 
			{
				if (!Character.isDigit(strTrack.charAt(i)) )
				{
					System.out.println("Incorrectly formatted tracking number");
					codeText="Incorrectly formed DataToEncode parameter";	
					return;
				}
			}
			//First lets check to make sure the Barcode Identifier (first to characters of tracking)
			//is correct.  Per spec, the first digit should be 0-9 and the second digit 0-4.
			//Since we already know we have a number, all we need to check is the second digit to make
			//it is less then or equal to 4
			if (Character.getNumericValue(strTrack.charAt(1)) >4) 
			{
				System.out.println("Incorrectly formatted Barcode Identifier");
				codeText="Incorrectly formed DataToEncode parameter";	
				return;
			}
			//Step one.  Convert the Routing Code
			//According to the spec, the routing code is converted as follows...
			//     If the routing code is 0 digits long, Value = 0
			//     If the routing code is 5 digits long, Value = (5-digit string converted to integer) + 1
			//     If the routing code is 9 digits long, Value = (9-digit string converted to integer) + 100000 + 1
			//     If the routing code is 11 digits long, Value = (11-digit string converted to integer) +
			//           1000000000 + 100000 + 1
			long iRoute = 0;

			if (strRoute.length() ==5) iRoute = (Integer.valueOf(strRoute.toString()).intValue()+1);
			else if (strRoute.length() ==9) iRoute =(Integer.valueOf(strRoute.toString()).intValue() +100000 +1);
			else if (strRoute.length() ==11) iRoute = (Integer.valueOf(strRoute.toString()).longValue()) +1000000000+100000+1;
			else if (strRoute.length() >0) 				
			{
				System.out.println("Incorrectly formed DataToEncode parameter");
				codeText="Incorrectly formed DataToEncode parameter";	
				return;
			}

			byteArray[12]= (int)(iRoute& 0x000000FF);
			byteArray[11]= (int)((iRoute >> 8)& 0x000000FF);
			byteArray[10]= (int)((iRoute >> 16)& 0x000000FF);
			byteArray[9]= (int)((iRoute >>24)& 0x000000FF);
			byteArray[8]= (int)((iRoute >> 32)& 0x000000FF);

			//Now convert the tracking code.
			//According to the spec, the tracking code is converted as follows...
			//  Multiply the Binary Data field by 10 decimal, and then add the first Tracking Code digit (left
			//   digit of Barcode Identifier).
			//  Multiply the Binary Data field by 5 decimal, and then add the second Tracking Code digit
			//   (right digit of Barcode Identifier, which is limited to value of 0 to 4).
			//	For each of the remaining 18 Tracking Code digits (from left to right, each of which can range
			//   from 0 to 9), multiply the Binary Data Field by 10 decimal, then add the Tracking Code digit.
			//
			MultiplyBytesByShort( byteArray, 13, (int) 10 );
			AddShortToBytes(byteArray, 13, Integer.valueOf(strTrack.substring(0,1)).intValue() );

			MultiplyBytesByShort( byteArray, 13, (int) 5 );
			AddShortToBytes( byteArray, 13,(Integer.valueOf(strTrack.substring(1,2)).intValue() ) );
			
			for ( int DigitIndex = 2; DigitIndex < 20; DigitIndex++ )
			{
				MultiplyBytesByShort( byteArray, 13, (int) 10 );
				AddShortToBytes( byteArray, 13, (Integer.valueOf(strTrack.substring(DigitIndex,DigitIndex+1)).intValue() ) );
			}

			//Step two. 
			//  An 11-bit CRC Frame
			// Check Sequence (FCS) value shall be generated by applying the Generator Polynomial
			// (0xF35) to the rightmost 102 bits of the Binary Data. The leftmost 2 bits of the leftmost
			// byte shall be excluded from the CRC calculation. This 11-bit FCS value shall be set
			// aside for later use.
			FrameCheckSequence11BitValue = GenerateCRC11FrameCheckSequence( byteArray );	
			//Now convert the binary data into codewords.
			//Per the spec...
			// 1  Divide Binary Data by 636, quotient replaces Binary Data, remainder is Codeword J
			// 2  Divide Binary Data by 1365, quotient replaces Binary Data, remainder is Codeword I
			// 3  Divide Binary Data by 1365, quotient replaces Binary Data, remainder is Codeword H
			// 4  Divide Binary Data by 1365, quotient replaces Binary Data, remainder is Codeword G
			// 5  Divide Binary Data by 1365, quotient replaces Binary Data, remainder is Codeword F
			// 6  Divide Binary Data by 1365, quotient replaces Binary Data, remainder is Codeword E
			// 7  Divide Binary Data by 1365, quotient replaces Binary Data, remainder is Codeword D
			// 8  Divide Binary Data by 1365, quotient replaces Binary Data, remainder is Codeword C
			// 9  Divide Binary Data by 1365, quotient replaces Binary Data, remainder is Codeword B
			// 10  Binary Data (should be value between 0 and 658) is Codeword A
			//GetNof13Table( 5, Table5of13ArrayPtr, NumberOf5of13TableEntries );
			if ( GetNof13Table( 5, Table5of13ArrayPtr ) != true )
			{
				System.out.println("Table 5 of 13 retrieve error");
				return;
			}
			// Get the 2 of 13 table we need 
			if ( GetNof13Table( 2, Table2of13ArrayPtr ) != true )
			{
				System.out.println("Table 2 of 13 retrieve error");
				return;
			}
			
			// Convert to base that allows 5 or 2 of 13 representation. 
			for ( int CodewordIndex = 0; CodewordIndex < 10; CodewordIndex++ )
			{

				CodewordArray[CodewordIndex][0]   = (double)( NumberOf5of13TableEntries + NumberOf2of13TableEntries);
				CodewordArray[CodewordIndex][1] = 0;

			}
			CodewordArray[0][0] = 659;
			CodewordArray[9][0] = 636;
			if ( ConvertFromBytesToMultiBase( 13, 10 ) != true )
			{
				System.out.println( "Codeword conversion failed");
				return;
			}

			if ( CodewordArray[0][1] >= 659 )
			{
				System.out.println("Codeword conversion failed1");
				return;
			}
			if ( CodewordArray[9][1] >= 636 )
			{
				System.out.println("Codeword conversion failed2");
				return;
			}

			CodewordArray[9][1] = CodewordArray[9][1] * 2;
		// Put the leftmost FCS bit into the leftmost codeword 
			if ( (FrameCheckSequence11BitValue >> 10)!=0 )
				CodewordArray[0][1] += 659;
			// Convert from codewords to 13 bit characters 
			for (int CharacterIndex = 0; CharacterIndex < 10; CharacterIndex++ )
			{
				if ( CodewordArray[CharacterIndex][1] >= NumberOf5of13TableEntries + NumberOf2of13TableEntries )
				{
					System.out.println("Character Range Error");
					return;
				}
				else if ( CodewordArray[CharacterIndex][1] >= NumberOf5of13TableEntries )
				{
					CharacterArray[CharacterIndex][0]   = 8192;
					CharacterArray[CharacterIndex][1] = Table2of13[(int)(CodewordArray[CharacterIndex][1] - NumberOf5of13TableEntries)];
				}
				else
				{
					CharacterArray[CharacterIndex][0]   = 8192;
					CharacterArray[CharacterIndex][1] = Table5of13[(int)CodewordArray[CharacterIndex][1]];
				}
			}

			for ( int CharacterIndex = 0; CharacterIndex < 10; CharacterIndex++ )
				if ( (FrameCheckSequence11BitValue & (1 << CharacterIndex))!=0 )
					CharacterArray[CharacterIndex][1] = ~ (int)CharacterArray[CharacterIndex][1] & 0x1FFF;


			// Map 13 bit characters to their positions within the barcode
			for ( int BarIndex = 0; BarIndex < 65; BarIndex++ )
			{
				BarTopArray[BarIndex]    = ((int)CharacterArray[BarTopCharacterIndexArray[BarIndex]][1]    >> BarTopCharacterShiftArray[BarIndex])    & 1;
				BarBottomArray[BarIndex] = ((int)CharacterArray[BarBottomCharacterIndexArray[BarIndex]][1] >> BarBottomCharacterShiftArray[BarIndex]) & 1;
			}

			//Convert the barcode to a string of characters representing the 4-state bars 
			for (int BarIndex = 0; BarIndex < 65; BarIndex++ )
				if ( BarTopArray[BarIndex] == 0 )
					if ( BarBottomArray[BarIndex] == 0 )
						UserDataEncoded.append('T');
					else
						UserDataEncoded.append('D');
				else
					if ( BarBottomArray[BarIndex] == 0 )
					UserDataEncoded.append('A');
				else
					UserDataEncoded.append('F');
			
			//format HRText according to spec
			codeText = (strTrack.toString() );
			
			if (strRoute.length()>1)codeText+=(strRoute.toString());
			String codeTextTotal = codeText;
			
			//codeText = UserDataEncoded.toString();
			if (strRoute.length()==11)
				codeText= codeText.substring(0,29) + " " +codeText.substring(29);
			if (strRoute.length()>=9)
				codeText= codeText.substring(0,25) + " " +codeText.substring(25);
			if (strRoute.length()>=5)
				codeText= codeText.substring(0,20) + " " +codeText.substring(20);
  		codeText= codeText.substring(0,11) + " " +codeText.substring(11);
			codeText= codeText.substring(0,5) + " " +codeText.substring(5);
			codeText= codeText.substring(0,2) + " " +codeText.substring(2);
			currentX=leftMarginPixels;
			
		
			
			int ArrayIndex = 0;
			// paint char in string
			for (int i = 0; i <= UserDataEncoded.length() - 1; i++)
			{				
      int postNetShortBarHeightPixels=(int) (postnetHeightShortBar*resolution);			
			int postNetTallBarHeightPixels = 3*postNetShortBarHeightPixels;
			float fltBarHeight = 0F;  
			double fltBarYCoord = 0D;
			if(UserDataEncoded.charAt(i) == 'F')
				{	//Calculate the Height of a Full Bar
					//In PLANET Barcodes 0 is short and 1 is tall
					fltBarYCoord = 0;
					fltBarHeight = postNetShortBarHeightPixels *3;
				}
				else if(UserDataEncoded.charAt(i) == 'D')
				{	//Calculate the Height of a SHORT Bar
					//In PLANET Barcodes 0 is short and 1 is tall
					fltBarYCoord = postNetShortBarHeightPixels;
					fltBarHeight = postNetShortBarHeightPixels *3;
				}
				else if(UserDataEncoded.charAt(i) == 'A')
				{	//Calculate the Height of a SHORT Bar
					//In PLANET Barcodes 0 is short and 1 is tall
					fltBarYCoord = 0;
					fltBarHeight = postNetShortBarHeightPixels *2;
				}
				else if(UserDataEncoded.charAt(i) == 'T')
				{	//Calculate the Height of a SHORT Bar
					//In PLANET Barcodes 0 is short and 1 is tall
					fltBarYCoord = postNetShortBarHeightPixels;
					fltBarHeight = postNetShortBarHeightPixels*2;
				}
				barHeightPixels = (int)fltBarHeight;
				paintChar2(g,"bw","11",(int)fltBarYCoord,paint);
			}

			}

		}//end of function Draw Demo
		
